@@ -200,6 +200,9 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("default-capabilities") {
 		config.DefaultCapabilities = StringSliceTrySplit(ctx, "default-capabilities")
 	}
+	if ctx.IsSet("add-inheritable-capabilities") {
+		config.AddInheritableCapabilities = ctx.Bool("add-inheritable-capabilities")
+	}
 	if ctx.IsSet("default-sysctls") {
 		config.DefaultSysctls = StringSliceTrySplit(ctx, "default-sysctls")
 	}
@@ -419,7 +422,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "conmon",
-			Usage:     fmt.Sprintf("Path to the conmon binary, used for monitoring the OCI runtime. Will be searched for using $PATH if empty. This option is deprecated, and will be removed in the future. (default: %q)", defConf.Conmon),
+			Usage:     "Path to the conmon binary, used for monitoring the OCI runtime. Will be searched for using $PATH if empty. This option is deprecated, and will be removed in the future.",
 			EnvVars:   []string{"CONTAINER_CONMON"},
 			TakesFile: true,
 		},
@@ -431,14 +434,14 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "listen",
-			Usage:     "Path to the CRI-O socket",
+			Usage:     "Path to the CRI-O socket.",
 			Value:     defConf.Listen,
 			EnvVars:   []string{"CONTAINER_LISTEN"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "stream-address",
-			Usage:   "Bind address for streaming socket",
+			Usage:   "Bind address for streaming socket.",
 			Value:   defConf.StreamAddress,
 			EnvVars: []string{"CONTAINER_STREAM_ADDRESS"},
 		},
@@ -450,21 +453,21 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "log",
-			Usage:     "Set the log file path where internal debug information is written",
+			Usage:     "Set the log file path where internal debug information is written.",
 			EnvVars:   []string{"CONTAINER_LOG"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "log-format",
 			Value:   "text",
-			Usage:   "Set the format used by logs: 'text' or 'json'",
+			Usage:   "Set the format used by logs: 'text' or 'json'.",
 			EnvVars: []string{"CONTAINER_LOG_FORMAT"},
 		},
 		&cli.StringFlag{
 			Name:    "log-level",
 			Aliases: []string{"l"},
 			Value:   "info",
-			Usage:   "Log messages above specified level: trace, debug, info, warn, error, fatal or panic",
+			Usage:   "Log messages above specified level: trace, debug, info, warn, error, fatal or panic.",
 			EnvVars: []string{"CONTAINER_LOG_LEVEL"},
 		},
 		&cli.StringFlag{
@@ -474,57 +477,57 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "log-dir",
-			Usage:     "Default log directory where all logs will go unless directly specified by the kubelet",
+			Usage:     "Default log directory where all logs will go unless directly specified by the kubelet.",
 			Value:     defConf.LogDir,
 			EnvVars:   []string{"CONTAINER_LOG_DIR"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "pause-command",
-			Usage:   "Path to the pause executable in the pause image",
+			Usage:   "Path to the pause executable in the pause image.",
 			Value:   defConf.PauseCommand,
 			EnvVars: []string{"CONTAINER_PAUSE_COMMAND"},
 		},
 		&cli.StringFlag{
 			Name:    "pause-image",
-			Usage:   "Image which contains the pause executable",
+			Usage:   "Image which contains the pause executable.",
 			Value:   defConf.PauseImage,
 			EnvVars: []string{"CONTAINER_PAUSE_IMAGE"},
 		},
 		&cli.StringFlag{
 			Name:      "pause-image-auth-file",
-			Usage:     fmt.Sprintf("Path to a config file containing credentials for --pause-image (default: %q)", defConf.PauseImageAuthFile),
+			Usage:     "Path to a config file containing credentials for --pause-image.",
 			EnvVars:   []string{"CONTAINER_PAUSE_IMAGE_AUTH_FILE"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "separate-pull-cgroup",
-			Usage:   fmt.Sprintf("[EXPERIMENTAL] Pull in new cgroup (default: %q)", defConf.SeparatePullCgroup),
+			Usage:   "[EXPERIMENTAL] Pull in new cgroup.",
 			EnvVars: []string{"PULL_IN_A_CGROUP"},
 		},
 		&cli.StringFlag{
 			Name:      "global-auth-file",
-			Usage:     fmt.Sprintf("Path to a file like /var/lib/kubelet/config.json holding credentials necessary for pulling images from secure registries (default: %q)", defConf.GlobalAuthFile),
+			Usage:     "Path to a file like /var/lib/kubelet/config.json holding credentials necessary for pulling images from secure registries.",
 			EnvVars:   []string{"CONTAINER_GLOBAL_AUTH_FILE"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "signature-policy",
-			Usage:     fmt.Sprintf("Path to signature policy JSON file. (default: %q, to use the system-wide default)", defConf.SignaturePolicyPath),
+			Usage:     "Path to signature policy JSON file.",
 			EnvVars:   []string{"CONTAINER_SIGNATURE_POLICY"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "root",
 			Aliases:   []string{"r"},
-			Usage:     "The CRI-O root directory",
+			Usage:     "The CRI-O root directory.",
 			Value:     defConf.Root,
 			EnvVars:   []string{"CONTAINER_ROOT"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "runroot",
-			Usage:     "The CRI-O state directory",
+			Usage:     "The CRI-O state directory.",
 			Value:     defConf.RunRoot,
 			EnvVars:   []string{"CONTAINER_RUNROOT"},
 			TakesFile: true,
@@ -532,13 +535,13 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		&cli.StringFlag{
 			Name:    "storage-driver",
 			Aliases: []string{"s"},
-			Usage:   fmt.Sprintf("OCI storage driver (default: %q)", defConf.Storage),
+			Usage:   "OCI storage driver.",
 			EnvVars: []string{"CONTAINER_STORAGE_DRIVER"},
 		},
 		&cli.StringSliceFlag{
 			Name:    "storage-opt",
 			Value:   cli.NewStringSlice(defConf.StorageOptions...),
-			Usage:   "OCI storage driver option",
+			Usage:   "OCI storage driver option.",
 			EnvVars: []string{"CONTAINER_STORAGE_OPT"},
 		},
 		&cli.StringSliceFlag{
@@ -559,12 +562,12 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		&cli.StringSliceFlag{
 			Name:    "registry",
 			Value:   cli.NewStringSlice(defConf.Registries...),
-			Usage:   "Registry to be prepended when pulling unqualified images, can be specified multiple times",
+			Usage:   "Registry to be prepended when pulling unqualified images. Can be specified multiple times.",
 			EnvVars: []string{"CONTAINER_REGISTRY"},
 		},
 		&cli.StringFlag{
 			Name:    "default-transport",
-			Usage:   "A prefix to prepend to image names that cannot be pulled as-is",
+			Usage:   "A prefix to prepend to image names that cannot be pulled as-is.",
 			Value:   defConf.DefaultTransport,
 			EnvVars: []string{"CONTAINER_DEFAULT_TRANSPORT"},
 		},
@@ -575,24 +578,24 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:    "default-runtime",
-			Usage:   "Default OCI runtime from the runtimes config",
+			Usage:   "Default OCI runtime from the runtimes config.",
 			Value:   defConf.DefaultRuntime,
 			EnvVars: []string{"CONTAINER_DEFAULT_RUNTIME"},
 		},
 		&cli.StringSliceFlag{
 			Name:    "runtimes",
-			Usage:   "OCI runtimes, format is runtime_name:runtime_path:runtime_root:runtime_type:privileged_without_host_devices:runtime_config_path",
+			Usage:   "OCI runtimes, format is 'runtime_name:runtime_path:runtime_root:runtime_type:privileged_without_host_devices:runtime_config_path'.",
 			EnvVars: []string{"CONTAINER_RUNTIMES"},
 		},
 		&cli.StringFlag{
 			Name:      "seccomp-profile",
-			Usage:     fmt.Sprintf("Path to the seccomp.json profile to be used as the runtime's default. If not specified, then the internal default seccomp profile will be used. (default: %q)", defConf.SeccompProfile),
+			Usage:     "Path to the seccomp.json profile to be used as the runtime's default. If not specified, then the internal default seccomp profile will be used.",
 			EnvVars:   []string{"CONTAINER_SECCOMP_PROFILE"},
 			TakesFile: true,
 		},
 		&cli.BoolFlag{
 			Name:    "seccomp-use-default-when-empty",
-			Usage:   "Use the default seccomp profile when an empty one is specified",
+			Usage:   "Use the default seccomp profile when an empty one is specified.",
 			EnvVars: []string{"CONTAINER_SECCOMP_USE_DEFAULT_WHEN_EMPTY"},
 			Value:   defConf.Seccomp().UseDefaultWhenEmpty(),
 		},
@@ -614,17 +617,18 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:  "rdt-config-file",
-			Usage: "Path to the RDT configuration file for configuring the resctrl pseudo-filesystem",
+			Usage: "Path to the RDT configuration file for configuring the resctrl pseudo-filesystem.",
 			Value: defConf.RdtConfigFile,
 		},
 		&cli.BoolFlag{
 			Name:    "selinux",
-			Usage:   fmt.Sprintf("Enable selinux support (default: %t)", defConf.SELinux),
+			Usage:   "Enable selinux support.",
 			EnvVars: []string{"CONTAINER_SELINUX"},
+			Value:   defConf.SELinux,
 		},
 		&cli.StringFlag{
 			Name:    "cgroup-manager",
-			Usage:   "cgroup manager (cgroupfs or systemd)",
+			Usage:   "cgroup manager (cgroupfs or systemd).",
 			Value:   defConf.CgroupManagerName,
 			EnvVars: []string{"CONTAINER_CGROUP_MANAGER"},
 		},
@@ -642,8 +646,9 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.BoolFlag{
 			Name:    "log-journald",
-			Usage:   fmt.Sprintf("Log to systemd journal (journald) in addition to kubernetes log file (default: %t)", defConf.LogToJournald),
+			Usage:   "Log to systemd journal (journald) in addition to kubernetes log file.",
 			EnvVars: []string{"CONTAINER_LOG_JOURNALD"},
+			Value:   defConf.LogToJournald,
 		},
 		&cli.StringFlag{
 			Name:    "cni-default-network",
@@ -653,7 +658,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "cni-config-dir",
-			Usage:     "CNI configuration files directory",
+			Usage:     "CNI configuration files directory.",
 			Value:     defConf.NetworkDir,
 			EnvVars:   []string{"CONTAINER_CNI_CONFIG_DIR"},
 			TakesFile: true,
@@ -661,7 +666,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		&cli.StringSliceFlag{
 			Name:    "cni-plugin-dir",
 			Value:   cli.NewStringSlice(defConf.PluginDir),
-			Usage:   "CNI plugin binaries directory",
+			Usage:   "CNI plugin binaries directory.",
 			EnvVars: []string{"CONTAINER_CNI_PLUGIN_DIR"},
 		},
 		&cli.StringFlag{
@@ -701,162 +706,176 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:      "default-mounts-file",
-			Usage:     fmt.Sprintf("Path to default mounts file (default: %q)", defConf.DefaultMountsFile),
+			Usage:     "Path to default mounts file.",
 			EnvVars:   []string{"CONTAINER_DEFAULT_MOUNTS_FILE"},
 			TakesFile: true,
 		},
 		&cli.StringSliceFlag{
 			Name:    "default-capabilities",
-			Usage:   "Capabilities to add to the containers",
+			Usage:   "Capabilities to add to the containers.",
 			EnvVars: []string{"CONTAINER_DEFAULT_CAPABILITIES"},
 			Value:   cli.NewStringSlice(defConf.DefaultCapabilities...),
 		},
+		&cli.BoolFlag{
+			Name:    "add-inheritable-capabilities",
+			Usage:   "Add capabilities to the inheritable set, as well as the default group of permitted, bounding and effective.",
+			EnvVars: []string{"CONTAINER_ADD_INHERITABLE_CAPABILITIES"},
+			Value:   defConf.AddInheritableCapabilities,
+		},
 		&cli.StringSliceFlag{
 			Name:    "default-sysctls",
-			Usage:   "Sysctls to add to the containers",
+			Usage:   "Sysctls to add to the containers.",
 			EnvVars: []string{"CONTAINER_DEFAULT_SYSCTLS"},
 			Value:   cli.NewStringSlice(defConf.DefaultSysctls...),
 		},
 		&cli.StringSliceFlag{
 			Name:    "default-ulimits",
-			Usage:   fmt.Sprintf("Ulimits to apply to containers by default (name=soft:hard) (default: %q)", defConf.DefaultUlimits),
+			Usage:   "Ulimits to apply to containers by default (name=soft:hard).",
 			EnvVars: []string{"CONTAINER_DEFAULT_ULIMITS"},
+			Value:   cli.NewStringSlice(defConf.DefaultUlimits...),
 		},
 		&cli.BoolFlag{
 			Name:    "profile",
-			Usage:   "Enable pprof remote profiler on localhost:6060",
+			Usage:   "Enable pprof remote profiler on localhost:6060.",
 			EnvVars: []string{"CONTAINER_PROFILE"},
 		},
 		&cli.StringFlag{
 			Name:    "profile-cpu",
-			Usage:   "Write a pprof CPU profile to the provided path",
+			Usage:   "Write a pprof CPU profile to the provided path.",
 			EnvVars: []string{"CONTAINER_PROFILE_CPU"},
 		},
 		&cli.StringFlag{
 			Name:    "profile-mem",
-			Usage:   "Write a pprof memory profile to the provided path",
+			Usage:   "Write a pprof memory profile to the provided path.",
 			EnvVars: []string{"CONTAINER_PROFILE_MEM"},
 		},
 		&cli.IntFlag{
 			Name:    "profile-port",
 			Value:   6060,
-			Usage:   "Port for the pprof profiler",
+			Usage:   "Port for the pprof profiler.",
 			EnvVars: []string{"CONTAINER_PROFILE_PORT"},
 		},
 		&cli.BoolFlag{
 			Name:    "enable-profile-unix-socket",
-			Usage:   "Enable pprof profiler on crio unix domain socket",
+			Usage:   "Enable pprof profiler on crio unix domain socket.",
 			EnvVars: []string{"ENABLE_PROFILE_UNIX_SOCKET"},
 		},
 		&cli.BoolFlag{
 			Name:    "enable-metrics",
-			Usage:   "Enable metrics endpoint for the server on localhost:9090",
+			Usage:   "Enable metrics endpoint for the server on localhost:9090.",
 			EnvVars: []string{"CONTAINER_ENABLE_METRICS"},
+			Value:   defConf.EnableMetrics,
 		},
 		&cli.IntFlag{
 			Name:    "metrics-port",
-			Value:   9090,
-			Usage:   "Port for the metrics endpoint",
+			Usage:   "Port for the metrics endpoint.",
 			EnvVars: []string{"CONTAINER_METRICS_PORT"},
+			Value:   defConf.MetricsPort,
 		},
 		&cli.StringFlag{
 			Name:    "metrics-socket",
-			Usage:   "Socket for the metrics endpoint",
+			Usage:   "Socket for the metrics endpoint.",
 			EnvVars: []string{"CONTAINER_METRICS_SOCKET"},
+			Value:   defConf.MetricsSocket,
 		},
 		&cli.StringFlag{
 			Name:    "metrics-cert",
-			Usage:   "Certificate for the secure metrics endpoint",
+			Usage:   "Certificate for the secure metrics endpoint.",
 			EnvVars: []string{"CONTAINER_METRICS_CERT"},
+			Value:   defConf.MetricsCert,
 		},
 		&cli.StringFlag{
 			Name:    "metrics-key",
-			Usage:   "Certificate key for the secure metrics endpoint",
+			Usage:   "Certificate key for the secure metrics endpoint.",
 			EnvVars: []string{"CONTAINER_METRICS_KEY"},
+			Value:   defConf.MetricsKey,
 		},
 		&cli.StringSliceFlag{
 			Name:    "metrics-collectors",
-			Usage:   "Enabled metrics collectors",
+			Usage:   "Enabled metrics collectors.",
 			Value:   cli.NewStringSlice(collectors.All().ToSlice()...),
 			EnvVars: []string{"CONTAINER_METRICS_COLLECTORS"},
 		},
 		&cli.BoolFlag{
 			Name:    "enable-tracing",
-			Usage:   "Enable OpenTelemetry trace data exporting",
+			Usage:   "Enable OpenTelemetry trace data exporting.",
 			EnvVars: []string{"CONTAINER_ENABLE_TRACING"},
+			Value:   defConf.EnableTracing,
 		},
 		&cli.IntFlag{
 			Name:    "tracing-sampling-rate-per-million",
 			Value:   defConf.TracingSamplingRatePerMillion,
-			Usage:   "Number of samples to collect per million OpenTelemetry spans",
+			Usage:   "Number of samples to collect per million OpenTelemetry spans.",
 			EnvVars: []string{"CONTAINER_TRACING_SAMPLING_RATE_PER_MILLION"},
 		},
 		&cli.StringFlag{
 			Name:    "tracing-endpoint",
 			Value:   defConf.TracingEndpoint,
-			Usage:   "Address on which the gRPC tracing collector will listen",
+			Usage:   "Address on which the gRPC tracing collector will listen.",
 			EnvVars: []string{"CONTAINER_TRACING_ENDPOINT"},
 		},
 		&cli.StringFlag{
 			Name:    "big-files-temporary-dir",
 			Usage:   `Path to the temporary directory to use for storing big files, used to store image blobs and data streams related to containers image management.`,
 			EnvVars: []string{"CONTAINER_BIG_FILES_TEMPORARY_DIR"},
+			Value:   defConf.BigFilesTemporaryDir,
 		},
 		&cli.BoolFlag{
 			Name:    "read-only",
-			Usage:   fmt.Sprintf("Setup all unprivileged containers to run as read-only. Automatically mounts tmpfs on `/run`, `/tmp` and `/var/tmp`. (default: %t)", defConf.ReadOnly),
+			Usage:   "Setup all unprivileged containers to run as read-only. Automatically mounts the containers' tmpfs on `/run`, `/tmp` and `/var/tmp`.",
 			EnvVars: []string{"CONTAINER_READ_ONLY"},
 		},
 		&cli.StringFlag{
 			Name:    "bind-mount-prefix",
-			Usage:   fmt.Sprintf("A prefix to use for the source of the bind mounts. This option would be useful if you were running CRI-O in a container. And had `/` mounted on `/host` in your container. Then if you ran CRI-O with the `--bind-mount-prefix=/host` option, CRI-O would add /host to any bind mounts it is handed over CRI. If Kubernetes asked to have `/var/lib/foobar` bind mounted into the container, then CRI-O would bind mount `/host/var/lib/foobar`. Since CRI-O itself is running in a container with `/` or the host mounted on `/host`, the container would end up with `/var/lib/foobar` from the host mounted in the container rather then `/var/lib/foobar` from the CRI-O container. (default: %q)", defConf.BindMountPrefix),
+			Usage:   "A prefix to use for the source of the bind mounts. This option would be useful if you were running CRI-O in a container. And had `/` mounted on `/host` in your container. Then if you ran CRI-O with the `--bind-mount-prefix=/host` option, CRI-O would add /host to any bind mounts it is handed over CRI. If Kubernetes asked to have `/var/lib/foobar` bind mounted into the container, then CRI-O would bind mount `/host/var/lib/foobar`. Since CRI-O itself is running in a container with `/` or the host mounted on `/host`, the container would end up with `/var/lib/foobar` from the host mounted in the container rather then `/var/lib/foobar` from the CRI-O container.",
 			EnvVars: []string{"CONTAINER_BIND_MOUNT_PREFIX"},
 		},
 		&cli.StringFlag{
 			Name:    "uid-mappings",
-			Usage:   fmt.Sprintf("Specify the UID mappings to use for the user namespace (default: %q)", defConf.UIDMappings),
+			Usage:   "Specify the UID mappings to use for the user namespace.",
 			Value:   "",
 			EnvVars: []string{"CONTAINER_UID_MAPPINGS"},
 		},
 		&cli.StringFlag{
 			Name:    "gid-mappings",
-			Usage:   fmt.Sprintf("Specify the GID mappings to use for the user namespace (default: %q)", defConf.GIDMappings),
+			Usage:   "Specify the GID mappings to use for the user namespace.",
 			Value:   "",
 			EnvVars: []string{"CONTAINER_GID_MAPPINGS"},
 		},
 		&cli.Int64Flag{
 			Name:    "minimum-mappable-uid",
-			Usage:   "Specify the lowest host UID which can be specified in mappings for a pod that will be run as a UID other than 0",
+			Usage:   "Specify the lowest host UID which can be specified in mappings for a pod that will be run as a UID other than 0.",
 			Value:   defConf.MinimumMappableUID,
 			EnvVars: []string{"CONTAINER_MINIMUM_MAPPABLE_UID"},
 		},
 		&cli.Int64Flag{
 			Name:    "minimum-mappable-gid",
-			Usage:   "Specify the lowest host GID which can be specified in mappings for a pod that will be run as a UID other than 0",
+			Usage:   "Specify the lowest host GID which can be specified in mappings for a pod that will be run as a UID other than 0.",
 			Value:   defConf.MinimumMappableGID,
 			EnvVars: []string{"CONTAINER_MINIMUM_MAPPABLE_GID"},
 		},
 		&cli.StringSliceFlag{
 			Name:    "allowed-devices",
-			Usage:   "Devices a user is allowed to specify with the \"io.kubernetes.cri-o.Devices\" allowed annotation",
+			Usage:   "Devices a user is allowed to specify with the \"io.kubernetes.cri-o.Devices\" allowed annotation.",
 			Value:   cli.NewStringSlice(defConf.AllowedDevices...),
 			EnvVars: []string{"CONTAINER_ALLOWED_DEVICES"},
 		},
 		&cli.StringSliceFlag{
 			Name:    "additional-devices",
-			Usage:   "Devices to add to the containers ",
+			Usage:   "Devices to add to the containers.",
 			Value:   cli.NewStringSlice(defConf.AdditionalDevices...),
 			EnvVars: []string{"CONTAINER_ADDITIONAL_DEVICES"},
 		},
 		&cli.StringSliceFlag{
 			Name:    "cdi-spec-dirs",
-			Usage:   "Directories to scan for CDI Spec files",
+			Usage:   "Directories to scan for CDI Spec files.",
 			Value:   cli.NewStringSlice(defConf.CDISpecDirs...),
 			EnvVars: []string{"CONTAINER_CDI_SPEC_DIRS"},
 		},
 		&cli.BoolFlag{
 			Name:  "device-ownership-from-security-context",
-			Usage: "Set devices' uid/gid ownership from runAsUser/runAsGroup",
+			Usage: "Set devices' uid/gid ownership from runAsUser/runAsGroup.",
+			Value: defConf.DeviceOwnershipFromSecurityContext,
 		},
 		&cli.StringSliceFlag{
 			Name:    "conmon-env",
@@ -867,92 +886,99 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		&cli.StringSliceFlag{
 			Name:    "default-env",
 			Value:   cli.NewStringSlice(defConf.DefaultEnv...),
-			Usage:   "Additional environment variables to set for all containers",
+			Usage:   "Additional environment variables to set for all containers.",
 			EnvVars: []string{"CONTAINER_DEFAULT_ENV"},
 		},
 		&cli.StringFlag{
 			Name:      "container-attach-socket-dir",
-			Usage:     "Path to directory for container attach sockets",
+			Usage:     "Path to directory for container attach sockets.",
 			Value:     defConf.ContainerAttachSocketDir,
 			EnvVars:   []string{"CONTAINER_ATTACH_SOCKET_DIR"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
-			Name:  "container-exits-dir",
-			Usage: "Path to directory in which container exit files are written to by conmon", Value: defConf.ContainerExitsDir,
+			Name:      "container-exits-dir",
+			Usage:     "Path to directory in which container exit files are written to by conmon.",
+			Value:     defConf.ContainerExitsDir,
 			EnvVars:   []string{"CONTAINER_EXITS_DIR"},
 			TakesFile: true,
 		},
 		&cli.Int64Flag{
 			Name:    "ctr-stop-timeout",
-			Usage:   "The minimal amount of time in seconds to wait before issuing a timeout regarding the proper termination of the container. The lowest possible value is 30s, whereas lower values are not considered by CRI-O",
+			Usage:   "The minimal amount of time in seconds to wait before issuing a timeout regarding the proper termination of the container. The lowest possible value is 30s, whereas lower values are not considered by CRI-O.",
 			Value:   defConf.CtrStopTimeout,
 			EnvVars: []string{"CONTAINER_STOP_TIMEOUT"},
 		},
 		&cli.IntFlag{
 			Name:    "grpc-max-recv-msg-size",
-			Usage:   "Maximum grpc receive message size in bytes",
+			Usage:   "Maximum grpc receive message size in bytes.",
 			Value:   defConf.GRPCMaxRecvMsgSize,
 			EnvVars: []string{"CONTAINER_GRPC_MAX_RECV_MSG_SIZE"},
 		},
 		&cli.IntFlag{
 			Name:    "grpc-max-send-msg-size",
-			Usage:   "Maximum grpc receive message size",
+			Usage:   "Maximum grpc receive message size.",
 			Value:   defConf.GRPCMaxSendMsgSize,
 			EnvVars: []string{"CONTAINER_GRPC_MAX_SEND_MSG_SIZE"},
 		},
 		&cli.BoolFlag{
 			Name:    "drop-infra-ctr",
-			Usage:   fmt.Sprintf("Determines whether pods are created without an infra container, when the pod is not using a pod level PID namespace (default: %v)", defConf.DropInfraCtr),
+			Usage:   "Determines whether pods are created without an infra container, when the pod is not using a pod level PID namespace.",
 			EnvVars: []string{"CONTAINER_DROP_INFRA_CTR"},
+			Value:   defConf.DropInfraCtr,
 		},
 		&cli.StringFlag{
-			Name:    "pinns-path",
-			Usage:   fmt.Sprintf("The path to find the pinns binary, which is needed to manage namespace lifecycle. Will be searched for in $PATH if empty (default: %q)", defConf.PinnsPath),
-			EnvVars: []string{"CONTAINER_PINNS_PATH"},
+			Name:      "pinns-path",
+			Usage:     "The path to find the pinns binary, which is needed to manage namespace lifecycle. Will be searched for in $PATH if empty.",
+			EnvVars:   []string{"CONTAINER_PINNS_PATH"},
+			Value:     defConf.PinnsPath,
+			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "namespaces-dir",
-			Usage:   "The directory where the state of the managed namespaces gets tracked. Only used when manage-ns-lifecycle is true",
+			Usage:   "The directory where the state of the managed namespaces gets tracked. Only used when manage-ns-lifecycle is true.",
 			Value:   defConf.NamespacesDir,
 			EnvVars: []string{"CONTAINER_NAMESPACES_DIR"},
 		},
 		&cli.BoolFlag{
 			Name:    "no-pivot",
-			Usage:   fmt.Sprintf("If true, the runtime will not use `pivot_root`, but instead use `MS_MOVE` (default: %v)", defConf.NoPivot),
+			Usage:   "If true, the runtime will not use `pivot_root`, but instead use `MS_MOVE`.",
 			EnvVars: []string{"CONTAINER_NO_PIVOT"},
+			Value:   defConf.NoPivot,
 		},
 		&cli.BoolFlag{
 			Name:    "stream-enable-tls",
-			Usage:   fmt.Sprintf("Enable encrypted TLS transport of the stream server (default: %v)", defConf.StreamEnableTLS),
+			Usage:   "Enable encrypted TLS transport of the stream server.",
 			EnvVars: []string{"CONTAINER_ENABLE_TLS"},
+			Value:   defConf.StreamEnableTLS,
 		},
 		&cli.StringFlag{
 			Name:      "stream-tls-ca",
-			Usage:     fmt.Sprintf("Path to the x509 CA(s) file used to verify and authenticate client communication with the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes (default: %q)", defConf.StreamTLSCA),
+			Usage:     "Path to the x509 CA(s) file used to verify and authenticate client communication with the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes.",
 			EnvVars:   []string{"CONTAINER_TLS_CA"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "stream-tls-cert",
-			Usage:     fmt.Sprintf("Path to the x509 certificate file used to serve the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes (default: %q)", defConf.StreamTLSCert),
+			Usage:     "Path to the x509 certificate file used to serve the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes.",
 			EnvVars:   []string{"CONTAINER_TLS_CERT"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "stream-tls-key",
-			Usage:     fmt.Sprintf("Path to the key file used to serve the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes (default: %q)", defConf.StreamTLSKey),
+			Usage:     "Path to the key file used to serve the encrypted stream. This file can change and CRI-O will automatically pick up the changes within 5 minutes.",
 			EnvVars:   []string{"CONTAINER_TLS_KEY"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:    "stream-idle-timeout",
-			Usage:   "Length of time until open streams terminate due to lack of activity",
+			Usage:   "Length of time until open streams terminate due to lack of activity.",
 			EnvVars: []string{"STREAM_IDLE_TIMEOUT"},
+			Value:   defConf.StreamIdleTimeout,
 		},
 		&cli.StringFlag{
 			Name:        "registries-conf",
-			Usage:       "path to the registries.conf file",
+			Usage:       "path to the registries.conf file.",
 			Destination: &defConf.SystemContext.SystemRegistriesConfPath,
 			Hidden:      true,
 			EnvVars:     []string{"CONTAINER_REGISTRIES_CONF"},
@@ -960,7 +986,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:        "registries-conf-dir",
-			Usage:       "path to the registries.conf.d directory",
+			Usage:       "path to the registries.conf.d directory.",
 			Destination: &defConf.SystemContext.SystemRegistriesConfDirPath,
 			Hidden:      true,
 			EnvVars:     []string{"CONTAINER_REGISTRIES_CONF_DIR"},
@@ -968,19 +994,19 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:   "address",
-			Usage:  "address used for the publish command",
+			Usage:  "address used for the publish command.",
 			Hidden: true,
 		},
 		&cli.StringFlag{
 			Name:      "version-file",
-			Usage:     "Location for CRI-O to lay down the temporary version file. It is used to check if crio wipe should wipe containers, which should always happen on a node reboot",
+			Usage:     "Location for CRI-O to lay down the temporary version file. It is used to check if crio wipe should wipe containers, which should always happen on a node reboot.",
 			Value:     defConf.VersionFile,
 			EnvVars:   []string{"CONTAINER_VERSION_FILE"},
 			TakesFile: true,
 		},
 		&cli.StringFlag{
 			Name:      "version-file-persist",
-			Usage:     "Location for CRI-O to lay down the persistent version file. It is used to check if crio wipe should wipe images, which should only happen when CRI-O has been upgraded",
+			Usage:     "Location for CRI-O to lay down the persistent version file. It is used to check if crio wipe should wipe images, which should only happen when CRI-O has been upgraded.",
 			Value:     defConf.VersionFile,
 			EnvVars:   []string{"CONTAINER_VERSION_FILE_PERSIST"},
 			TakesFile: true,
@@ -993,12 +1019,13 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.StringFlag{
 			Name:    "infra-ctr-cpuset",
-			Usage:   "CPU set to run infra containers, if not specified CRI-O will use all online CPUs to run infra containers (default: '').",
+			Usage:   "CPU set to run infra containers, if not specified CRI-O will use all online CPUs to run infra containers.",
 			EnvVars: []string{"CONTAINER_INFRA_CTR_CPUSET"},
+			Value:   defConf.InfraCtrCPUSet,
 		},
 		&cli.StringFlag{
 			Name:      "clean-shutdown-file",
-			Usage:     "Location for CRI-O to lay down the clean shutdown file. It indicates whether we've had time to sync changes to disk before shutting down. If not found, crio wipe will clear the storage directory",
+			Usage:     "Location for CRI-O to lay down the clean shutdown file. It indicates whether we've had time to sync changes to disk before shutting down. If not found, crio wipe will clear the storage directory.",
 			Value:     defConf.CleanShutdownFile,
 			EnvVars:   []string{"CONTAINER_CLEAN_SHUTDOWN_FILE"},
 			TakesFile: true,
@@ -1017,7 +1044,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.BoolFlag{
 			Name:    "enable-criu-support",
-			Usage:   "Enable CRIU integration, requires that the criu binary is available in $PATH. (default: '')",
+			Usage:   "Enable CRIU integration, requires that the criu binary is available in $PATH.",
 			EnvVars: []string{"CONTAINER_ENABLE_CRIU_SUPPORT"},
 			Value:   false,
 		},
