@@ -2,19 +2,24 @@ package oci
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"strconv"
+	"strings"
 	"syscall"
 	"time"
 
+	metadata "github.com/checkpoint-restore/checkpointctl/lib"
 	"github.com/containernetworking/plugins/pkg/ns"
 	conmonconfig "github.com/containers/conmon/runner/config"
+	"github.com/containers/podman/v4/pkg/annotations"
+	"github.com/containers/podman/v4/pkg/checkpoint/crutils"
+	"github.com/containers/podman/v4/pkg/criu"
 	"github.com/containers/storage/pkg/pools"
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/log"
@@ -25,7 +30,7 @@ import (
 	"github.com/fsnotify/fsnotify"
 	json "github.com/json-iterator/go"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
+	"github.com/opencontainers/runtime-tools/generate"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
@@ -80,7 +85,7 @@ type exitCodeInfo struct {
 }
 
 // CreateContainer creates a container.
-func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupParent string) (retErr error) {
+func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupParent string, restore bool) (retErr error) {
 	if c.Spoofed() {
 		return nil
 	}
@@ -88,11 +93,11 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 	var stderrBuf bytes.Buffer
 	parentPipe, childPipe, err := newPipe()
 	if err != nil {
-		return fmt.Errorf("error creating socket pair: %v", err)
+		return fmt.Errorf("error creating socket pair: %w", err)
 	}
 	childStartPipe, parentStartPipe, err := newPipe()
 	if err != nil {
-		return fmt.Errorf("error creating socket pair: %v", err)
+		return fmt.Errorf("error creating socket pair: %w", err)
 	}
 	defer parentPipe.Close()
 	defer parentStartPipe.Close()
@@ -134,6 +139,31 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 		}
 		args = append(args, "-i")
 	}
+	if restore {
+		logrus.Debugf("Restore is true %v", restore)
+		args = append(args, "--restore", c.CheckpointPath())
+		if c.Spec().Process.SelinuxLabel != "" {
+			args = append(
+				args,
+				"--runtime-opt",
+				fmt.Sprintf(
+					"--lsm-profile=selinux:%s",
+					c.Spec().Process.SelinuxLabel,
+				),
+			)
+		}
+		if c.Spec().Linux.MountLabel != "" {
+			args = append(
+				args,
+				"--runtime-opt",
+				fmt.Sprintf(
+					"--lsm-mount-context=%s",
+					c.Spec().Linux.MountLabel,
+				),
+			)
+		}
+	}
+
 	logrus.WithFields(logrus.Fields{
 		"args": args,
 	}).Debugf("running conmon: %s", r.handler.MonitorPath)
@@ -156,6 +186,12 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 	if v, found := os.LookupEnv("XDG_RUNTIME_DIR"); found {
 		cmd.Env = append(cmd.Env, fmt.Sprintf("XDG_RUNTIME_DIR=%s", v))
 	}
+	if restore {
+		// The CRIU binary is usually in /usr/sbin/criu
+		if v, found := os.LookupEnv("PATH"); found {
+			cmd.Env = append(cmd.Env, fmt.Sprintf("PATH=/usr/sbin/%s", v))
+		}
+	}
 
 	err = cmd.Start()
 	if err != nil {
@@ -177,13 +213,13 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 				killErr := cmd.Process.Kill()
 				waitErr := cmd.Wait()
 				if killErr != nil {
-					retErr = errors.Wrapf(retErr, "failed to kill %+v after failing with", killErr)
+					retErr = fmt.Errorf("failed to kill %+v after failing with: %w", killErr, retErr)
 				}
 				// Per https://pkg.go.dev/os#ProcessState.ExitCode, the exit code is -1 when the process died because
 				// of a signal. We expect this in this case, as we've just killed it with a signal. Don't append the
 				// error in this case to reduce noise.
 				if exitErr, ok := waitErr.(*exec.ExitError); !ok || exitErr.ExitCode() != -1 {
-					retErr = errors.Wrapf(retErr, "failed to wait %+v after failing with", waitErr)
+					retErr = fmt.Errorf("failed to wait %+v after failing with: %w", waitErr, retErr)
 				}
 			}
 		}()
@@ -242,12 +278,16 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 	select {
 	case ss := <-ch:
 		if ss.err != nil {
-			return fmt.Errorf("error reading container (probably exited) json message: %v", ss.err)
+			return fmt.Errorf("error reading container (probably exited) json message: %w", ss.err)
 		}
 		log.Debugf(ctx, "Received container pid: %d", ss.si.Pid)
 		pid = ss.si.Pid
 		if ss.si.Pid == -1 {
 			if ss.si.Message != "" {
+				if restore {
+					log.Errorf(ctx, "Container restore error: %s", ss.si.Message)
+					return fmt.Errorf("container restore failed: %s", ss.si.Message)
+				}
 				log.Errorf(ctx, "Container creation error: %s", ss.si.Message)
 				return fmt.Errorf("container create failed: %s", ss.si.Message)
 			}
@@ -276,9 +316,7 @@ func (r *runtimeOCI) StartContainer(ctx context.Context, c *Container) error {
 		return nil
 	}
 
-	if _, err := utils.ExecCmd(
-		r.handler.RuntimePath, rootFlag, r.root, "start", c.ID(),
-	); err != nil {
+	if _, err := r.runtimeCmd("start", c.ID()); err != nil {
 		return err
 	}
 	c.state.Started = time.Now()
@@ -292,7 +330,7 @@ func prepareExec() (pidFileName string, parentPipe, childPipe *os.File, _ error)
 		return "", nil, nil, err
 	}
 
-	pidFile, err := ioutil.TempFile("", "pidfile")
+	pidFile, err := os.CreateTemp("", "pidfile")
 	if err != nil {
 		parentPipe.Close()
 		childPipe.Close()
@@ -360,8 +398,8 @@ func (r *runtimeOCI) ExecContainer(ctx context.Context, c *Container, cmd []stri
 	}
 	defer os.RemoveAll(processFile)
 
-	args := []string{rootFlag, r.root, "exec"}
-	args = append(args, "--process", processFile, c.ID())
+	args := r.defaultRuntimeArgs()
+	args = append(args, "exec", "--process", processFile, c.ID())
 	execCmd := cmdrunner.Command(r.handler.RuntimePath, args...) // nolint: gosec
 	if v, found := os.LookupEnv("XDG_RUNTIME_DIR"); found {
 		execCmd.Env = append(execCmd.Env, fmt.Sprintf("XDG_RUNTIME_DIR=%s", v))
@@ -410,7 +448,7 @@ func (r *runtimeOCI) ExecContainer(ctx context.Context, c *Container, cmd []stri
 		if r != nil {
 			if err := r.Close(); err != nil {
 				if waitErr := execCmd.Wait(); waitErr != nil {
-					return errors.Wrap(err, waitErr.Error())
+					return fmt.Errorf("%v: %w", waitErr, err)
 				}
 				return err
 			}
@@ -458,7 +496,7 @@ func (r *runtimeOCI) ExecSyncContainer(ctx context.Context, c *Container, comman
 		}
 	}()
 
-	logFile, err := ioutil.TempFile("", "crio-log-"+c.ID())
+	logFile, err := os.CreateTemp("", "crio-log-"+c.ID())
 	if err != nil {
 		return nil, &ExecSyncError{
 			ExitCode: -1,
@@ -495,6 +533,9 @@ func (r *runtimeOCI) ExecSyncContainer(ctx context.Context, c *Container, comman
 	if timeout > 0 {
 		args = append(args, "-T", fmt.Sprintf("%d", timeout))
 	}
+	if r.config.CgroupManager().IsSystemd() {
+		args = append(args, "-s")
+	}
 
 	processFile, err := prepareProcessExec(c, command, c.terminal)
 	if err != nil {
@@ -564,13 +605,13 @@ func (r *runtimeOCI) ExecSyncContainer(ctx context.Context, c *Container, comman
 				killErr := cmd.Process.Kill()
 				waitErr := cmd.Wait()
 				if killErr != nil {
-					retErr = errors.Wrapf(retErr, "failed to kill %+v after failing with", killErr)
+					retErr = fmt.Errorf("failed to kill %+v after failing with: %w", killErr, retErr)
 				}
 				// Per https://pkg.go.dev/os#ProcessState.ExitCode, the exit code is -1 when the process died because
 				// of a signal. We expect this in this case, as we've just killed it with a signal. Don't append the
 				// error in this case to reduce noise.
 				if exitErr, ok := waitErr.(*exec.ExitError); !ok || exitErr.ExitCode() != -1 {
-					retErr = errors.Wrapf(retErr, "failed to wait %+v after failing with", waitErr)
+					retErr = fmt.Errorf("failed to wait %+v after failing with: %w", waitErr, retErr)
 				}
 			}
 		}()
@@ -699,6 +740,9 @@ func TruncateAndReadFile(ctx context.Context, path string, size int64) ([]byte,
 
 // UpdateContainer updates container resources
 func (r *runtimeOCI) UpdateContainer(ctx context.Context, c *Container, res *rspec.LinuxResources) error {
+	c.opLock.Lock()
+	defer c.opLock.Unlock()
+
 	if c.Spoofed() {
 		return nil
 	}
@@ -718,7 +762,7 @@ func (r *runtimeOCI) UpdateContainer(ctx context.Context, c *Container, res *rsp
 	cmd.Stdin = bytes.NewReader(jsonResources)
 
 	if err := cmd.Run(); err != nil {
-		return fmt.Errorf("updating resources for container %q failed: %v %v (%v)", c.ID(), stderr.String(), stdout.String(), err)
+		return fmt.Errorf("updating resources for container %q failed: %v %v: %w", c.ID(), stderr.String(), stdout.String(), err)
 	}
 	return nil
 }
@@ -775,7 +819,7 @@ func WaitContainerStop(ctx context.Context, c *Container, timeout time.Duration,
 				return err
 			}
 			if err := Kill(pid); err != nil {
-				return fmt.Errorf("failed to kill process: %v", err)
+				return fmt.Errorf("failed to kill process: %w", err)
 			}
 			killed = true
 		case newTimeout := <-c.stopTimeoutChan:
@@ -784,7 +828,7 @@ func WaitContainerStop(ctx context.Context, c *Container, timeout time.Duration,
 			newTargetTime := time.Now().Add(newTimeout)
 
 			// but only if it's earlier
-			if !newTargetTime.Before(targetTime) {
+			if newTargetTime.After(targetTime) {
 				continue
 			}
 
@@ -807,16 +851,14 @@ func (r *runtimeOCI) StopContainer(ctx context.Context, c *Container, timeout in
 		return nil
 	}
 	defer func() {
-		if retErr != nil {
-			// Failed to stop, set stopping to false.
-			// Otherwise, we won't actually
-			// attempt to stop when a new request comes in,
-			// even though we're not actively stopping anymore.
-			// Also, close the stopStoppingChan to tell
-			// routines waiting to change the stop timeout to give up.
-			close(c.stopStoppingChan)
-			c.SetAsNotStopping()
-		}
+		// Failed to stop, set stopping to false.
+		// Otherwise, we won't actually
+		// attempt to stop when a new request comes in,
+		// even though we're not actively stopping anymore.
+		// Also, close the stopStoppingChan to tell
+		// routines waiting to change the stop timeout to give up.
+		close(c.stopStoppingChan)
+		c.SetAsNotStopping()
 	}()
 
 	c.opLock.Lock()
@@ -839,9 +881,7 @@ func (r *runtimeOCI) StopContainer(ctx context.Context, c *Container, timeout in
 	}
 
 	if timeout > 0 {
-		if _, err := utils.ExecCmd(
-			r.handler.RuntimePath, rootFlag, r.root, "kill", c.ID(), c.GetStopSignal(),
-		); err != nil {
+		if _, err := r.runtimeCmd("kill", c.ID(), c.GetStopSignal()); err != nil {
 			checkProcessGone(c)
 		}
 		err := WaitContainerStop(ctx, c, time.Duration(timeout)*time.Second, true)
@@ -851,9 +891,7 @@ func (r *runtimeOCI) StopContainer(ctx context.Context, c *Container, timeout in
 		log.Warnf(ctx, "Stopping container %v with stop signal timed out: %v", c.ID(), err)
 	}
 
-	if _, err := utils.ExecCmd(
-		r.handler.RuntimePath, rootFlag, r.root, "kill", c.ID(), "KILL",
-	); err != nil {
+	if _, err := r.runtimeCmd("kill", c.ID(), "KILL"); err != nil {
 		checkProcessGone(c)
 	}
 
@@ -877,7 +915,7 @@ func (r *runtimeOCI) DeleteContainer(ctx context.Context, c *Container) error {
 		return nil
 	}
 
-	_, err := utils.ExecCmd(r.handler.RuntimePath, rootFlag, r.root, "delete", "--force", c.ID())
+	_, err := r.runtimeCmd("delete", "--force", c.ID())
 	return err
 }
 
@@ -885,19 +923,19 @@ func updateContainerStatusFromExitFile(c *Container) error {
 	exitFilePath := c.exitFilePath()
 	fi, err := os.Stat(exitFilePath)
 	if err != nil {
-		return errors.Wrapf(err, "failed to find container exit file for %s", c.ID())
+		return fmt.Errorf("failed to find container exit file for %s: %w", c.ID(), err)
 	}
 	c.state.Finished, err = getFinishedTime(fi)
 	if err != nil {
-		return errors.Wrap(err, "failed to get finished time")
+		return fmt.Errorf("failed to get finished time: %w", err)
 	}
-	statusCodeStr, err := ioutil.ReadFile(exitFilePath)
+	statusCodeStr, err := os.ReadFile(exitFilePath)
 	if err != nil {
-		return errors.Wrap(err, "failed to read exit file")
+		return fmt.Errorf("failed to read exit file: %w", err)
 	}
-	statusCode, err := strconv.Atoi(string(statusCodeStr))
+	statusCode, err := strconv.ParseInt(string(statusCodeStr), 10, 32)
 	if err != nil {
-		return errors.Wrap(err, "status code conversion failed")
+		return fmt.Errorf("status code conversion failed: %w", err)
 	}
 	c.state.ExitCode = utils.Int32Ptr(int32(statusCode))
 	return nil
@@ -918,11 +956,7 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 	}
 
 	stateCmd := func() (*ContainerState, bool, error) {
-		cmd := cmdrunner.Command(r.handler.RuntimePath, rootFlag, r.root, "state", c.ID()) // nolint: gosec
-		if v, found := os.LookupEnv("XDG_RUNTIME_DIR"); found {
-			cmd.Env = append(cmd.Env, fmt.Sprintf("XDG_RUNTIME_DIR=%s", v))
-		}
-		out, err := cmd.Output()
+		out, err := r.runtimeCmd("state", c.ID())
 		if err != nil {
 			// there are many code paths that could lead to have a bad state in the
 			// underlying runtime.
@@ -931,7 +965,7 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 			// We always populate the fields below so kube can restart/reschedule
 			// containers failing.
 			if exitErr, isExitError := err.(*exec.ExitError); isExitError {
-				log.Errorf(ctx, "Failed to update container state for %s: stdout: %s, stderr: %s", c.ID(), string(out), string(exitErr.Stderr))
+				log.Errorf(ctx, "Failed to update container state for %s: stdout: %s, stderr: %s", c.ID(), out, string(exitErr.Stderr))
 			} else {
 				log.Errorf(ctx, "Failed to update container state for %s: %v", c.ID(), err)
 			}
@@ -943,7 +977,7 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 			return nil, true, nil
 		}
 		state := *c.state
-		if err := json.NewDecoder(bytes.NewBuffer(out)).Decode(&state); err != nil {
+		if err := json.NewDecoder(strings.NewReader(out)).Decode(&state); err != nil {
 			return &state, false, fmt.Errorf("failed to decode container status for %s: %s", c.ID(), err)
 		}
 		return &state, false, nil
@@ -963,7 +997,6 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 	// release the lock before waiting
 	c.opLock.Unlock()
 	exitFilePath := c.exitFilePath()
-	var fi os.FileInfo
 	err = kwait.ExponentialBackoff(
 		kwait.Backoff{
 			Duration: 500 * time.Millisecond,
@@ -971,8 +1004,7 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 			Steps:    6,
 		},
 		func() (bool, error) {
-			var err error
-			fi, err = os.Stat(exitFilePath)
+			_, err := os.Stat(exitFilePath)
 			if err != nil {
 				// wait longer
 				return false, nil
@@ -992,20 +1024,10 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 	if err != nil {
 		log.Warnf(ctx, "Failed to find container exit file for %v: %v", c.ID(), err)
 	} else {
-		c.state.Finished, err = getFinishedTime(fi)
-		if err != nil {
-			return fmt.Errorf("failed to get finished time: %v", err)
+		if err := updateContainerStatusFromExitFile(c); err != nil {
+			return err
 		}
-		statusCodeStr, err := ioutil.ReadFile(exitFilePath)
-		if err != nil {
-			return errors.Wrap(err, "failed to read exit file: %v")
-		}
-		statusCode, err := strconv.Atoi(string(statusCodeStr))
-		if err != nil {
-			return fmt.Errorf("status code conversion failed: %v", err)
-		}
-		c.state.ExitCode = utils.Int32Ptr(int32(statusCode))
-		log.Debugf(ctx, "Found exit code for %s: %d", c.ID(), statusCode)
+		log.Debugf(ctx, "Found exit code for %s: %d", c.ID(), *c.state.ExitCode)
 	}
 
 	oomFilePath := filepath.Join(c.bundlePath, "oom")
@@ -1018,7 +1040,12 @@ func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) er
 		// Collect metric by container name
 		metrics.Instance().MetricContainersOOMCountTotalInc(c.Name())
 	}
-
+	// If this container had a node level PID namespace, then any children processes will be leaked to init.
+	// Eventually, the processes will get cleaned up when the pod cgroup is cleaned by the kubelet,
+	// but this situation is atypical and should be avoided.
+	if c.nodeLevelPIDNamespace() {
+		return r.signalContainer(c, syscall.SIGKILL, true)
+	}
 	return nil
 }
 
@@ -1031,7 +1058,7 @@ func (r *runtimeOCI) PauseContainer(ctx context.Context, c *Container) error {
 		return nil
 	}
 
-	_, err := utils.ExecCmd(r.handler.RuntimePath, rootFlag, r.root, "pause", c.ID())
+	_, err := r.runtimeCmd("pause", c.ID())
 	return err
 }
 
@@ -1044,7 +1071,7 @@ func (r *runtimeOCI) UnpauseContainer(ctx context.Context, c *Container) error {
 		return nil
 	}
 
-	_, err := utils.ExecCmd(r.handler.RuntimePath, rootFlag, r.root, "resume", c.ID())
+	_, err := r.runtimeCmd("resume", c.ID())
 	return err
 }
 
@@ -1065,12 +1092,21 @@ func (r *runtimeOCI) SignalContainer(ctx context.Context, c *Container, sig sysc
 	}
 
 	if unix.SignalName(sig) == "" {
-		return errors.Errorf("unable to find signal %s", sig.String())
+		return fmt.Errorf("unable to find signal %s", sig.String())
 	}
 
-	_, err := utils.ExecCmd(
-		r.handler.RuntimePath, rootFlag, r.root, "kill", c.ID(), strconv.Itoa(int(sig)),
-	)
+	return r.signalContainer(c, sig, false)
+}
+
+func (r *runtimeOCI) signalContainer(c *Container, sig syscall.Signal, all bool) error {
+	args := []string{
+		"kill",
+	}
+	if all {
+		args = append(args, "-a")
+	}
+	args = append(args, c.ID(), strconv.Itoa(int(sig)))
+	_, err := r.runtimeCmd(args...)
 	return err
 }
 
@@ -1083,7 +1119,7 @@ func (r *runtimeOCI) AttachContainer(ctx context.Context, c *Container, inputStr
 	controlPath := filepath.Join(c.BundlePath(), "ctl")
 	controlFile, err := os.OpenFile(controlPath, os.O_WRONLY, 0)
 	if err != nil {
-		return fmt.Errorf("failed to open container ctl file: %v", err)
+		return fmt.Errorf("failed to open container ctl file: %w", err)
 	}
 	defer controlFile.Close()
 
@@ -1098,7 +1134,7 @@ func (r *runtimeOCI) AttachContainer(ctx context.Context, c *Container, inputStr
 	attachSocketPath := filepath.Join(r.config.ContainerAttachSocketDir, c.ID(), "attach")
 	conn, err := net.DialUnix("unixpacket", nil, &net.UnixAddr{Name: attachSocketPath, Net: "unixpacket"})
 	if err != nil {
-		return fmt.Errorf("failed to connect to container %s attach socket: %v", c.ID(), err)
+		return fmt.Errorf("failed to connect to container %s attach socket: %w", c.ID(), err)
 	}
 	defer conn.Close()
 
@@ -1177,7 +1213,7 @@ func (r *runtimeOCI) PortForwardContainer(ctx context.Context, c *Container, net
 		d.FallbackDelay = -1
 		conn, err := d.Dial("tcp", fmt.Sprintf("localhost:%d", port))
 		if err != nil {
-			return errors.Wrapf(err, "failed to connect to localhost:%d inside namespace %s", port, c.ID())
+			return fmt.Errorf("failed to connect to localhost:%d inside namespace %s: %w", port, c.ID(), err)
 		}
 		defer conn.Close()
 
@@ -1234,8 +1270,8 @@ func (r *runtimeOCI) PortForwardContainer(ctx context.Context, c *Container, net
 
 		return errFwd
 	}); err != nil {
-		return errors.Wrapf(
-			err, "port forward into network namespace %q", netNsPath,
+		return fmt.Errorf(
+			"port forward into network namespace %q: %w", netNsPath, err,
 		)
 	}
 
@@ -1252,13 +1288,13 @@ func (r *runtimeOCI) ReopenContainerLog(ctx context.Context, c *Container) error
 	controlPath := filepath.Join(c.BundlePath(), "ctl")
 	controlFile, err := os.OpenFile(controlPath, os.O_WRONLY, 0)
 	if err != nil {
-		return fmt.Errorf("failed to open container ctl file: %v", err)
+		return fmt.Errorf("failed to open container ctl file: %w", err)
 	}
 	defer controlFile.Close()
 
 	watcher, err := fsnotify.NewWatcher()
 	if err != nil {
-		return fmt.Errorf("failed to create new watch: %v", err)
+		return fmt.Errorf("failed to create new watch: %w", err)
 	}
 	defer watcher.Close()
 
@@ -1279,7 +1315,7 @@ func (r *runtimeOCI) ReopenContainerLog(ctx context.Context, c *Container) error
 					}
 				}
 			case err := <-watcher.Errors:
-				errorCh <- fmt.Errorf("watch error for container log reopen %v: %v", c.ID(), err)
+				errorCh <- fmt.Errorf("watch error for container log reopen %v: %w", c.ID(), err)
 				close(errorCh)
 				return
 			}
@@ -1315,7 +1351,7 @@ func (r *runtimeOCI) ReopenContainerLog(ctx context.Context, c *Container) error
 // prepareProcessExec returns the path of the process.json used in runc exec -p
 // caller is responsible for removing the returned file, if prepareProcessExec succeeds
 func prepareProcessExec(c *Container, cmd []string, tty bool) (processFile string, retErr error) {
-	f, err := ioutil.TempFile("", "exec-process-")
+	f, err := os.CreateTemp("", "exec-process-")
 	if err != nil {
 		return "", err
 	}
@@ -1342,12 +1378,262 @@ func prepareProcessExec(c *Container, cmd []string, tty bool) (processFile strin
 		return "", err
 	}
 
-	if err := ioutil.WriteFile(processFile, processJSON, 0o644); err != nil {
+	if err := os.WriteFile(processFile, processJSON, 0o644); err != nil {
 		return "", err
 	}
 	return processFile, nil
 }
 
+// ReadConmonPidFile attempts to read conmon's pid from its pid file
+// This function makes no verification that this file should exist
+// it is up to the caller to verify that this container has a conmon
+func ReadConmonPidFile(c *Container) (int, error) {
+	contents, err := os.ReadFile(c.conmonPidFilePath())
+	if err != nil {
+		return -1, err
+	}
+	// Convert it to an int
+	conmonPID, err := strconv.Atoi(string(contents))
+	if err != nil {
+		return -1, err
+	}
+	return conmonPID, nil
+}
+
 func (c *Container) conmonPidFilePath() string {
 	return filepath.Join(c.bundlePath, "conmon-pidfile")
 }
+
+// runtimeCmd executes a command with args and returns its output as a string along
+// with an error, if any
+func (r *runtimeOCI) runtimeCmd(args ...string) (string, error) {
+	runtimeArgs := append(r.defaultRuntimeArgs(), args...)
+	cmd := cmdrunner.Command(r.handler.RuntimePath, runtimeArgs...)
+	var stdout bytes.Buffer
+	var stderr bytes.Buffer
+	cmd.Stdout = &stdout
+	cmd.Stderr = &stderr
+	if v, found := os.LookupEnv("XDG_RUNTIME_DIR"); found {
+		cmd.Env = append(cmd.Env, fmt.Sprintf("XDG_RUNTIME_DIR=%s", v))
+	}
+
+	err := cmd.Run()
+	if err != nil {
+		return "", fmt.Errorf("`%v %v` failed: %v %v: %w", r.handler.RuntimePath, strings.Join(runtimeArgs, " "), stderr.String(), stdout.String(), err)
+	}
+
+	return stdout.String(), nil
+}
+
+func (r *runtimeOCI) defaultRuntimeArgs() []string {
+	args := []string{rootFlag, r.root}
+	if r.config.CgroupManager().IsSystemd() {
+		args = append(args, "--systemd-cgroup")
+	}
+	return args
+}
+
+// CheckpointContainer checkpoints a container.
+func (r *runtimeOCI) CheckpointContainer(ctx context.Context, c *Container, specgen *rspec.Spec, leaveRunning bool) error {
+	c.opLock.Lock()
+	defer c.opLock.Unlock()
+
+	if err := r.checkpointRestoreSupported(); err != nil {
+		return err
+	}
+
+	// Once CRIU infects the process in the container with the
+	// parasite, the parasite also wants to write to the log
+	// file which is outside of the container. Giving the log file
+	// the label of the container enables logging for the parasite.
+	if err := crutils.CRCreateFileWithLabel(
+		c.Dir(),
+		metadata.DumpLogFile,
+		specgen.Linux.MountLabel,
+	); err != nil {
+		return err
+	}
+
+	// workPath will be used to store dump.log and stats-dump
+	workPath := c.Dir()
+	// imagePath is used by CRIU to store the actual checkpoint files
+	imagePath := c.CheckpointPath()
+
+	logrus.Debugf("Writing checkpoint to %s", imagePath)
+	logrus.Debugf("Writing checkpoint logs to %s", workPath)
+	args := []string{}
+	args = append(
+		args,
+		"checkpoint",
+		"--image-path",
+		imagePath,
+		"--work-path",
+		workPath,
+	)
+	if leaveRunning {
+		args = append(args, "--leave-running")
+	}
+
+	args = append(args, c.ID())
+
+	_, err := r.runtimeCmd(args...)
+	if err != nil {
+		return fmt.Errorf("running %q %q failed: %w", r.handler.RuntimePath, args, err)
+	}
+
+	if !leaveRunning {
+		c.state.Status = ContainerStateStopped
+		c.state.ExitCode = utils.Int32Ptr(0)
+		c.state.Finished = time.Now()
+	}
+
+	return nil
+}
+
+// RestoreContainer restores a container.
+func (r *runtimeOCI) RestoreContainer(ctx context.Context, c *Container, sbSpec *rspec.Spec, infraPid int, cgroupParent string) error {
+	if err := r.checkpointRestoreSupported(); err != nil {
+		return err
+	}
+
+	// Let's try to stat() CRIU's inventory file. If it does not exist, it makes
+	// no sense to try a restore. This is a minimal check if a checkpoint exist.
+	if _, err := os.Stat(filepath.Join(c.CheckpointPath(), "inventory.img")); os.IsNotExist(err) {
+		return fmt.Errorf("a complete checkpoint for this container cannot be found, cannot restore: %w", err)
+	}
+
+	// remove conmon files
+	attachFile := filepath.Join(c.BundlePath(), "attach")
+	if err := os.Remove(attachFile); err != nil && !os.IsNotExist(err) {
+		return fmt.Errorf("error removing container %s attach file: %w", c.ID(), err)
+	}
+
+	ctlFile := filepath.Join(c.BundlePath(), "ctl")
+	if err := os.Remove(ctlFile); err != nil && !os.IsNotExist(err) {
+		return fmt.Errorf("error removing container %s ctl file: %w", c.ID(), err)
+	}
+
+	winszFile := filepath.Join(c.BundlePath(), "winsz")
+	if err := os.Remove(winszFile); err != nil && !os.IsNotExist(err) {
+		return fmt.Errorf("error removing container %s winsz file: %w", c.ID(), err)
+	}
+
+	// Figure out if this container will be restored in another sandbox
+	oldSbID := c.Sandbox()
+	if oldSbID == "" {
+		return fmt.Errorf("failed to detect sandbox of to be restored container %s", c.ID())
+	}
+	newSbID := sbSpec.Annotations[annotations.SandboxID]
+	if newSbID == "" {
+		return fmt.Errorf("failed to detect destination sandbox of to be restored container %s", c.ID())
+	}
+
+	// Get config.json to adapt for restore (mostly annotations for restore in another sandbox)
+	configFile := filepath.Join(c.BundlePath(), "config.json")
+	specgen, err := generate.NewFromFile(configFile)
+	if err != nil {
+		return err
+	}
+
+	if oldSbID != newSbID {
+		// The container will be restored in another (not the original) sandbox
+		// Adapt to namespaces of the new sandbox
+		for i, n := range specgen.Config.Linux.Namespaces {
+			if n.Path == "" {
+				// The namespace in the original container did not point to
+				// an existing interface. Leave it as it is.
+				continue
+			}
+			for _, on := range sbSpec.Linux.Namespaces {
+				if on.Type == n.Type {
+					var nsPath string
+					if n.Type == rspec.NetworkNamespace {
+						// Type for network namespaces is 'network'.
+						// The kernel link is 'net'.
+						nsPath = fmt.Sprintf("/proc/%d/ns/%s", infraPid, "net")
+					} else {
+						nsPath = fmt.Sprintf("/proc/%d/ns/%s", infraPid, n.Type)
+					}
+					specgen.Config.Linux.Namespaces[i].Path = nsPath
+					break
+				}
+			}
+		}
+
+		// Update Sandbox Name
+		specgen.AddAnnotation(annotations.SandboxName, sbSpec.Annotations[annotations.Name])
+		// Update Sandbox ID
+		specgen.AddAnnotation(annotations.SandboxID, newSbID)
+
+		// Update Name
+		ctrMetadata := types.ContainerMetadata{}
+		err = json.Unmarshal([]byte(sbSpec.Annotations[annotations.Metadata]), &ctrMetadata)
+		if err != nil {
+			return err
+		}
+		ctrName := ctrMetadata.Name
+
+		podMetadata := types.PodSandboxMetadata{}
+		err = json.Unmarshal([]byte(specgen.Config.Annotations[annotations.Metadata]), &podMetadata)
+		if err != nil {
+			return err
+		}
+		uid := podMetadata.Uid
+		mData := fmt.Sprintf("k8s_%s_%s_%s_%s0", ctrName, sbSpec.Annotations[annotations.KubeName], sbSpec.Annotations[annotations.Namespace], uid)
+		specgen.AddAnnotation(annotations.Name, mData)
+
+		c.SetSandbox(newSbID)
+
+		saveOptions := generate.ExportOptions{}
+		if err := specgen.SaveToFile(configFile, saveOptions); err != nil {
+			return err
+		}
+	}
+
+	c.state.InitPid = 0
+	c.state.InitStartTime = ""
+
+	// It is possible to tell runc to place the CRIU log files
+	// at a custom location '--work-path'. But for restoring a
+	// container we are not calling runc directly but conmon, which
+	// then calls runc. It would be possible to change conmon to
+	// also have the log file in the same location as during
+	// checkpointing, but it is not really that important right now.
+	if err := crutils.CRCreateFileWithLabel(
+		c.BundlePath(),
+		metadata.RestoreLogFile,
+		specgen.Config.Linux.MountLabel,
+	); err != nil {
+		return err
+	}
+
+	if err := r.CreateContainer(ctx, c, cgroupParent, true); err != nil {
+		return err
+	}
+
+	// Once the container is restored, update the metadata
+	// 1. Container is running again
+	c.state.Status = ContainerStateRunning
+	// 2. Update PID of the container (without that stopping will fail)
+	pid, err := ReadConmonPidFile(c)
+	if err != nil {
+		return err
+	}
+	c.state.Pid = pid
+	// 3. Reset ExitCode (also needed for stopping)
+	c.state.ExitCode = nil
+	// 4. Set start time
+	c.state.Started = time.Now()
+
+	return nil
+}
+
+func (r *runtimeOCI) checkpointRestoreSupported() error {
+	if !criu.CheckForCriu(criu.PodCriuVersion) {
+		return fmt.Errorf("checkpoint/restore requires at least CRIU %d", criu.PodCriuVersion)
+	}
+	if !crutils.CRRuntimeSupportsCheckpointRestore(r.handler.RuntimePath) {
+		return fmt.Errorf("configured runtime does not support checkpoint/restore")
+	}
+	return nil
+}
