@@ -2,13 +2,15 @@ package runtimehandlerhooks
 
 import (
 	"context"
+	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"os/exec"
+	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
+	"time"
 
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -18,9 +20,9 @@ import (
 	"github.com/cri-o/cri-o/utils/cmdrunner"
 	"github.com/opencontainers/runc/libcontainer/cgroups"
 	"github.com/opencontainers/runc/libcontainer/cgroups/systemd"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/fields"
+	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
 )
 
@@ -36,10 +38,13 @@ const (
 const (
 	annotationTrue       = "true"
 	annotationDisable    = "disable"
+	annotationEnable     = "enable"
 	schedDomainDir       = "/proc/sys/kernel/sched_domain"
 	cgroupMountPoint     = "/sys/fs/cgroup"
 	irqBalanceBannedCpus = "IRQBALANCE_BANNED_CPUS"
 	irqBalancedName      = "irqbalance"
+	sysCPUDir            = "/sys/devices/system/cpu"
+	sysCPUSaveDir        = "/var/run/crio/cpu"
 )
 
 // HighPerformanceHooks used to run additional hooks that will configure a system for the latency sensitive workloads
@@ -65,9 +70,8 @@ func (h *HighPerformanceHooks) PreStart(ctx context.Context, c *oci.Container, s
 
 	// disable the CPU load balancing for the container CPUs
 	if shouldCPULoadBalancingBeDisabled(s.Annotations()) {
-		log.Infof(ctx, "Disable cpu load balancing for container %q", c.ID())
-		if err := setCPUSLoadBalancing(c, false, schedDomainDir); err != nil {
-			return errors.Wrap(err, "set CPU load balancing")
+		if err := setCPUSLoadBalancingWithRetry(ctx, c, false); err != nil {
+			return fmt.Errorf("set CPU load balancing: %w", err)
 		}
 	}
 
@@ -75,7 +79,7 @@ func (h *HighPerformanceHooks) PreStart(ctx context.Context, c *oci.Container, s
 	if shouldIRQLoadBalancingBeDisabled(s.Annotations()) {
 		log.Infof(ctx, "Disable irq smp balancing for container %q", c.ID())
 		if err := setIRQLoadBalancing(c, false, IrqSmpAffinityProcFile, h.irqBalanceConfigFile); err != nil {
-			return errors.Wrap(err, "set IRQ load balancing")
+			return fmt.Errorf("set IRQ load balancing: %w", err)
 		}
 	}
 
@@ -87,7 +91,35 @@ func (h *HighPerformanceHooks) PreStart(ctx context.Context, c *oci.Container, s
 			return err
 		}
 		if err := setCPUQuota(cpuMountPoint, s.CgroupParent(), c, false); err != nil {
-			return errors.Wrap(err, "set CPU CFS quota")
+			return fmt.Errorf("set CPU CFS quota: %w", err)
+		}
+	}
+
+	// Configure c-states for the container CPUs.
+	if configure, value := shouldCStatesBeConfigured(s.Annotations()); configure {
+		log.Infof(ctx, "Configure c-states for container %q to %q", c.ID(), value)
+		switch value {
+		case annotationEnable:
+			// Enable all c-states.
+			if err := setCPUPMQOSResumeLatency(c, "0"); err != nil {
+				return fmt.Errorf("set CPU PM QOS resume latency: %w", err)
+			}
+		case annotationDisable:
+			// Lock the c-state to C0.
+			if err := setCPUPMQOSResumeLatency(c, "n/a"); err != nil {
+				return fmt.Errorf("set CPU PM QOS resume latency: %w", err)
+			}
+		default:
+			return fmt.Errorf("invalid annotation value %s", value)
+		}
+	}
+
+	// Configure cpu freq governor for the container CPUs.
+	if configure, value := shouldFreqGovernorBeConfigured(s.Annotations()); configure {
+		log.Infof(ctx, "Configure cpu freq governor for container %q to %q", c.ID(), value)
+		// Set the cpu freq governor to specified value.
+		if err := setCPUFreqGovernor(c, value); err != nil {
+			return fmt.Errorf("set CPU scaling governor: %w", err)
 		}
 	}
 
@@ -112,20 +144,38 @@ func (h *HighPerformanceHooks) PreStop(ctx context.Context, c *oci.Container, s
 
 	// enable the CPU load balancing for the container CPUs
 	if shouldCPULoadBalancingBeDisabled(s.Annotations()) {
-		if err := setCPUSLoadBalancing(c, true, schedDomainDir); err != nil {
-			return errors.Wrap(err, "set CPU load balancing")
+		if err := setCPUSLoadBalancingWithRetry(ctx, c, true); err != nil {
+			return fmt.Errorf("set CPU load balancing: %w", err)
 		}
 	}
 
 	// enable the IRQ smp balancing for the container CPUs
 	if shouldIRQLoadBalancingBeDisabled(s.Annotations()) {
 		if err := setIRQLoadBalancing(c, true, IrqSmpAffinityProcFile, h.irqBalanceConfigFile); err != nil {
-			return errors.Wrap(err, "set IRQ load balancing")
+			return fmt.Errorf("set IRQ load balancing: %w", err)
 		}
 	}
 
 	// no need to reverse the cgroup CPU CFS quota setting as the pod cgroup will be deleted anyway
 
+	// Restore the c-state configuration for the container CPUs (only do this when the annotation is
+	// present - without the annotation we do not modify the c-state).
+	if configure, _ := shouldCStatesBeConfigured(s.Annotations()); configure {
+		// Restore the original resume latency value.
+		if err := setCPUPMQOSResumeLatency(c, ""); err != nil {
+			return fmt.Errorf("set CPU PM QOS resume latency: %w", err)
+		}
+	}
+
+	// Restore the cpu freq governor for the container CPUs (only do this when the annotation is
+	// present - without the annotation we do not modify the governor).
+	if configure, _ := shouldFreqGovernorBeConfigured(s.Annotations()); configure {
+		// Restore the original scaling governor.
+		if err := setCPUFreqGovernor(c, ""); err != nil {
+			return fmt.Errorf("set CPU scaling governor: %w", err)
+		}
+	}
+
 	return nil
 }
 
@@ -156,6 +206,16 @@ func shouldIRQLoadBalancingBeDisabled(annotations fields.Set) bool {
 		annotations[crioannotations.IRQLoadBalancingAnnotation] == annotationDisable
 }
 
+func shouldCStatesBeConfigured(annotations fields.Set) (present bool, value string) {
+	value, present = annotations[crioannotations.CPUCStatesAnnotation]
+	return
+}
+
+func shouldFreqGovernorBeConfigured(annotations fields.Set) (present bool, value string) {
+	value, present = annotations[crioannotations.CPUFreqGovernorAnnotation]
+	return
+}
+
 func annotationValueDeprecationWarning(annotation string) string {
 	return fmt.Sprintf("The usage of the annotation %q with value %q will be deprecated under 1.21", annotation, "true")
 }
@@ -172,13 +232,32 @@ func isContainerRequestWholeCPU(c *oci.Container) bool {
 	return *(c.Spec().Linux.Resources.CPU.Shares)%1024 == 0
 }
 
+func setCPUSLoadBalancingWithRetry(ctx context.Context, c *oci.Container, enable bool) error {
+	log.Infof(ctx, "Disable cpu load balancing for container %q", c.ID())
+	// it is possible to have errors during reading or writing to sched_domain files because
+	// that kernel rebuilds it with updated values
+	// the retry will not fix it for 100% but should reduce the possibility for failures to minimum
+	// TODO: re-visit once we will have some more acceptable cgroups hierarchy to disable CPU load balancing
+	// correctly via cgroups, see -https://bugzilla.redhat.com/show_bug.cgi?id=1946801
+	return wait.PollImmediate(time.Second, 5*time.Second, func() (bool, error) {
+		if err := setCPUSLoadBalancing(c, enable, schedDomainDir); err != nil {
+			if os.IsNotExist(err) {
+				log.Errorf(ctx, "Failed to set CPU load balancing: %v", err)
+				return false, nil
+			}
+			return false, err
+		}
+		return true, nil
+	})
+}
+
 func setCPUSLoadBalancing(c *oci.Container, enable bool, schedDomainDir string) error {
 	lspec := c.Spec().Linux
 	if lspec == nil ||
 		lspec.Resources == nil ||
 		lspec.Resources.CPU == nil ||
 		lspec.Resources.CPU.Cpus == "" {
-		return errors.Errorf("find container %s CPUs", c.ID())
+		return fmt.Errorf("find container %s CPUs", c.ID())
 	}
 
 	cpus, err := cpuset.Parse(lspec.Resources.CPU.Cpus)
@@ -195,7 +274,7 @@ func setCPUSLoadBalancing(c *oci.Container, enable bool, schedDomainDir string)
 			if !info.Mode().IsRegular() || info.Name() != "flags" {
 				return nil
 			}
-			content, err := ioutil.ReadFile(path)
+			content, err := os.ReadFile(path)
 			if err != nil {
 				return err
 			}
@@ -217,7 +296,7 @@ func setCPUSLoadBalancing(c *oci.Container, enable bool, schedDomainDir string)
 				newContent = strconv.Itoa(flags & 32766)
 			}
 
-			return ioutil.WriteFile(path, []byte(newContent), 0o644)
+			return os.WriteFile(path, []byte(newContent), 0o644)
 		})
 		if err != nil {
 			return err
@@ -233,10 +312,10 @@ func setIRQLoadBalancing(c *oci.Container, enable bool, irqSmpAffinityFile, irqB
 		lspec.Resources == nil ||
 		lspec.Resources.CPU == nil ||
 		lspec.Resources.CPU.Cpus == "" {
-		return errors.Errorf("find container %s CPUs", c.ID())
+		return fmt.Errorf("find container %s CPUs", c.ID())
 	}
 
-	content, err := ioutil.ReadFile(irqSmpAffinityFile)
+	content, err := os.ReadFile(irqSmpAffinityFile)
 	if err != nil {
 		return err
 	}
@@ -245,7 +324,7 @@ func setIRQLoadBalancing(c *oci.Container, enable bool, irqSmpAffinityFile, irqB
 	if err != nil {
 		return err
 	}
-	if err := ioutil.WriteFile(irqSmpAffinityFile, []byte(newIRQSMPSetting), 0o644); err != nil {
+	if err := os.WriteFile(irqSmpAffinityFile, []byte(newIRQSMPSetting), 0o644); err != nil {
 		return err
 	}
 
@@ -318,17 +397,199 @@ func setCPUQuota(cpuMountPoint, parentDir string, c *oci.Container, enable bool)
 
 	if enable {
 		// there should have no use case to get here, as the pod cgroup will be deleted when the pod end
-		if err := ioutil.WriteFile(cfsQuotaPath, []byte("0"), 0o644); err != nil {
+		if err := os.WriteFile(cfsQuotaPath, []byte("0"), 0o644); err != nil {
 			return err
 		}
-		if err := ioutil.WriteFile(parentCfsQuotaPath, []byte("0"), 0o644); err != nil {
+		if err := os.WriteFile(parentCfsQuotaPath, []byte("0"), 0o644); err != nil {
 			return err
 		}
 	} else {
-		if err := ioutil.WriteFile(cfsQuotaPath, []byte("-1"), 0o644); err != nil {
+		if err := os.WriteFile(cfsQuotaPath, []byte("-1"), 0o644); err != nil {
 			return err
 		}
-		if err := ioutil.WriteFile(parentCfsQuotaPath, []byte("-1"), 0o644); err != nil {
+		if err := os.WriteFile(parentCfsQuotaPath, []byte("-1"), 0o644); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+// setCPUPMQOSResumeLatency sets the pm_qos_resume_latency_us for a cpu and stores the original
+// value so it can be restored later. If the latency is an empty string, the original latency
+// value is restored.
+func setCPUPMQOSResumeLatency(c *oci.Container, latency string) error {
+	return doSetCPUPMQOSResumeLatency(c, latency, sysCPUDir, sysCPUSaveDir)
+}
+
+// doSetCPUPMQOSResumeLatency facilitates unit testing by allowing the directories to be specified as parameters.
+func doSetCPUPMQOSResumeLatency(c *oci.Container, latency, cpuDir, cpuSaveDir string) error {
+	lspec := c.Spec().Linux
+	if lspec == nil ||
+		lspec.Resources == nil ||
+		lspec.Resources.CPU == nil ||
+		lspec.Resources.CPU.Cpus == "" {
+		return fmt.Errorf("find container %s CPUs", c.ID())
+	}
+
+	cpus, err := cpuset.Parse(lspec.Resources.CPU.Cpus)
+	if err != nil {
+		return err
+	}
+
+	for _, cpu := range cpus.ToSlice() {
+		latencyFile := fmt.Sprintf("%s/cpu%d/power/pm_qos_resume_latency_us", cpuDir, cpu)
+		cpuPowerSaveDir := fmt.Sprintf("%s/cpu%d/power", cpuSaveDir, cpu)
+		latencyFileOrig := path.Join(cpuPowerSaveDir, "pm_qos_resume_latency_us")
+
+		if latency != "" {
+			// Retrieve the current latency.
+			latencyOrig, err := os.ReadFile(latencyFile)
+			if err != nil {
+				return err
+			}
+
+			// Save the current latency so we can restore it later.
+			err = os.MkdirAll(cpuPowerSaveDir, 0o750)
+			if err != nil {
+				return err
+			}
+			err = os.WriteFile(latencyFileOrig, latencyOrig, 0o644)
+			if err != nil {
+				return err
+			}
+
+			// Update the pm_qos_resume_latency_us.
+			err = os.WriteFile(latencyFile, []byte(latency), 0o644)
+			if err != nil {
+				return err
+			}
+
+			continue
+		}
+
+		// Retrieve the original latency.
+		latencyOrig, err := os.ReadFile(latencyFileOrig)
+		if err != nil {
+			if errors.Is(err, os.ErrNotExist) {
+				// The latency may have already been restored by a previous invocation of the hook.
+				return nil
+			}
+			return err
+		}
+
+		// Restore the original latency.
+		err = os.WriteFile(latencyFile, latencyOrig, 0o644)
+		if err != nil {
+			return err
+		}
+
+		// Remove the saved latency.
+		err = os.Remove(latencyFileOrig)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+// isCPUGovernorSupported checks whether the cpu governor is supported for the specified cpu.
+func isCPUGovernorSupported(governor, cpuDir string, cpu int) error {
+	// Get available cpu scaling governors.
+	availGovernorFile := fmt.Sprintf("%s/cpu%d/cpufreq/scaling_available_governors", cpuDir, cpu)
+	availGovernors, err := os.ReadFile(availGovernorFile)
+	if err != nil {
+		return err
+	}
+
+	// Is the scaling governor supported?
+	for _, availableGovernor := range strings.Fields(string(availGovernors)) {
+		if availableGovernor == governor {
+			return nil
+		}
+	}
+
+	return fmt.Errorf("governor %s not available for cpu %d", governor, cpu)
+}
+
+// setCPUFreqGovernor sets the scaling_governor for a cpu and stores the original
+// value so it can be restored later. If the governor is an empty string, the original
+// scaling_governor value is restored.
+func setCPUFreqGovernor(c *oci.Container, governor string) error {
+	return doSetCPUFreqGovernor(c, governor, sysCPUDir, sysCPUSaveDir)
+}
+
+// doSetCPUFreqGovernor facilitates unit testing by allowing the directories to be specified as parameters.
+func doSetCPUFreqGovernor(c *oci.Container, governor, cpuDir, cpuSaveDir string) error {
+	lspec := c.Spec().Linux
+	if lspec == nil ||
+		lspec.Resources == nil ||
+		lspec.Resources.CPU == nil ||
+		lspec.Resources.CPU.Cpus == "" {
+		return fmt.Errorf("find container %s CPUs", c.ID())
+	}
+
+	cpus, err := cpuset.Parse(lspec.Resources.CPU.Cpus)
+	if err != nil {
+		return err
+	}
+
+	for _, cpu := range cpus.ToSlice() {
+		governorFile := fmt.Sprintf("%s/cpu%d/cpufreq/scaling_governor", cpuDir, cpu)
+		cpuFreqSaveDir := fmt.Sprintf("%s/cpu%d/cpufreq", cpuSaveDir, cpu)
+		governorFileOrig := path.Join(cpuFreqSaveDir, "scaling_governor")
+
+		if governor != "" {
+			// Retrieve the current scaling governor.
+			governorOrig, err := os.ReadFile(governorFile)
+			if err != nil {
+				return err
+			}
+
+			// Is the scaling governor supported?
+			if err := isCPUGovernorSupported(governor, cpuDir, cpu); err != nil {
+				return err
+			}
+
+			// Save the current governor so we can restore it later.
+			err = os.MkdirAll(cpuFreqSaveDir, 0o750)
+			if err != nil {
+				return err
+			}
+			err = os.WriteFile(governorFileOrig, governorOrig, 0o644)
+			if err != nil {
+				return err
+			}
+
+			// Update the governor.
+			err = os.WriteFile(governorFile, []byte(governor), 0o644)
+			if err != nil {
+				return err
+			}
+
+			continue
+		}
+
+		// Retrieve the original scaling governor.
+		governorOrig, err := os.ReadFile(governorFileOrig)
+		if err != nil {
+			if errors.Is(err, os.ErrNotExist) {
+				// The governor may have already been restored by a previous invocation of the hook.
+				return nil
+			}
+			return err
+		}
+
+		// Restore the original governor.
+		err = os.WriteFile(governorFile, governorOrig, 0o644)
+		if err != nil {
+			return err
+		}
+
+		// Remove the saved governor.
+		err = os.Remove(governorFileOrig)
+		if err != nil {
 			return err
 		}
 	}
@@ -338,7 +599,7 @@ func setCPUQuota(cpuMountPoint, parentDir string, c *oci.Container, enable bool)
 
 // RestoreIrqBalanceConfig restores irqbalance service with original banned cpu mask settings
 func RestoreIrqBalanceConfig(irqBalanceConfigFile, irqBannedCPUConfigFile, irqSmpAffinityProcFile string) error {
-	content, err := ioutil.ReadFile(irqSmpAffinityProcFile)
+	content, err := os.ReadFile(irqSmpAffinityProcFile)
 	if err != nil {
 		return err
 	}
@@ -372,7 +633,7 @@ func RestoreIrqBalanceConfig(irqBalanceConfigFile, irqBannedCPUConfigFile, irqSm
 		return nil
 	}
 
-	content, err = ioutil.ReadFile(irqBannedCPUConfigFile)
+	content, err = os.ReadFile(irqBannedCPUConfigFile)
 	if err != nil {
 		return err
 	}
