@@ -1,37 +1,36 @@
 package version
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
 	"reflect"
 	"runtime"
+	"runtime/debug"
 	"strings"
 	"text/tabwriter"
 
-	"github.com/blang/semver"
+	"github.com/blang/semver/v4"
 	"github.com/containers/common/pkg/apparmor"
 	"github.com/containers/common/pkg/seccomp"
 	"github.com/google/renameio"
 	json "github.com/json-iterator/go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 )
 
 // Version is the version of the build.
-const Version = "1.24.6"
+const Version = "1.25.1"
 
 // Variables injected during build-time
 var (
-	gitCommit    string   // sha1 from git, output of $(git rev-parse HEAD)
-	gitTreeState string   // state of git tree, either "clean" or "dirty"
-	buildDate    string   // build date in ISO8601 format, output of $(date -u +'%Y-%m-%dT%H:%M:%SZ')
-	buildTags    []string // tags to be used during build time
+	buildDate string // build date in ISO8601 format, output of $(date -u +'%Y-%m-%dT%H:%M:%SZ')
 )
 
 type Info struct {
 	Version         string   `json:"version,omitempty"`
 	GitCommit       string   `json:"gitCommit,omitempty"`
+	GitCommitDate   string   `json:"gitCommitDate,omitempty"`
 	GitTreeState    string   `json:"gitTreeState,omitempty"`
 	BuildDate       string   `json:"buildDate,omitempty"`
 	GoVersion       string   `json:"goVersion,omitempty"`
@@ -39,8 +38,10 @@ type Info struct {
 	Platform        string   `json:"platform,omitempty"`
 	Linkmode        string   `json:"linkmode,omitempty"`
 	BuildTags       []string `json:"buildTags,omitempty"`
+	LDFlags         string   `json:"ldFlags,omitempty"`
 	SeccompEnabled  bool     `json:"seccompEnabled"`
 	AppArmorEnabled bool     `json:"appArmorEnabled"`
+	Dependencies    []string `json:"dependencies,omitempty"`
 }
 
 // ShouldCrioWipe opens the version file, and parses it and the version string
@@ -59,19 +60,19 @@ func shouldCrioWipe(versionFileName, versionString string) (bool, error) {
 	}
 	versionBytes, err := os.ReadFile(versionFileName)
 	if err != nil {
-		return true, errors.Errorf("version file %s not found: %v", versionFileName, err)
+		return true, err
 	}
 
 	// parse the version that was laid down by a previous invocation of crio
 	var oldVersion semver.Version
 	if err := oldVersion.UnmarshalJSON(versionBytes); err != nil {
-		return true, errors.Errorf("version file %s malformatted: %v", versionFileName, err)
+		return true, fmt.Errorf("version file %s malformatted: %w", versionFileName, err)
 	}
 
 	// parse the version of the current binary
 	newVersion, err := parseVersionConstant(versionString, "")
 	if err != nil {
-		return true, errors.Errorf("version constant %s malformatted: %v", versionString, err)
+		return true, fmt.Errorf("version constant %s malformatted: %w", versionString, err)
 	}
 
 	// in every case that the minor and major version are out of sync,
@@ -84,13 +85,13 @@ func shouldCrioWipe(versionFileName, versionString string) (bool, error) {
 // file is the location of the old version file
 // gitCommit is the current git commit version. It will be added to the file
 // to aid in debugging, but will not be used to compare versions
-func WriteVersionFile(file string) error {
-	return writeVersionFile(file, gitCommit, Version)
+func (i *Info) WriteVersionFile(file string) error {
+	return writeVersionFile(file, i.GitCommit, Version)
 }
 
 // LogVersion logs the version and git information of this build
-func LogVersion() {
-	logrus.Infof("Starting CRI-O, version: %s, git: %v(%s)", Version, gitCommit, gitTreeState)
+func (i *Info) LogVersion() {
+	logrus.Infof("Starting CRI-O, version: %s, git: %v(%s)", Version, i.GitCommit, i.GitTreeState)
 }
 
 // writeVersionFile is an internal function for testing purposes
@@ -138,10 +139,54 @@ func parseVersionConstant(versionString, gitCommit string) (*semver.Version, err
 	return &v, nil
 }
 
-func Get() *Info {
+func Get(verbose bool) (*Info, error) {
+	info, ok := debug.ReadBuildInfo()
+	if !ok {
+		return nil, errors.New("unable to retrieve build info")
+	}
+
+	const unknown = "unknown"
+	gitCommit := unknown
+	gitTreeState := "clean"
+	gitCommitDate := unknown
+	buildTags := []string{}
+	ldFlags := unknown
+
+	for _, s := range info.Settings {
+		switch s.Key {
+		case "vcs.revision":
+			gitCommit = s.Value
+
+		case "vcs.modified":
+			if s.Value == "true" {
+				gitTreeState = "dirty"
+			}
+
+		case "vcs.time":
+			gitCommitDate = s.Value
+
+		case "-tags":
+			buildTags = strings.Split(s.Value, ",")
+
+		case "-ldflags":
+			ldFlags = s.Value
+		}
+	}
+
+	dependencies := []string{}
+	if verbose {
+		for _, d := range info.Deps {
+			dependencies = append(
+				dependencies,
+				fmt.Sprintf("%s %s %s", d.Path, d.Version, d.Sum),
+			)
+		}
+	}
+
 	return &Info{
 		Version:         Version,
 		GitCommit:       gitCommit,
+		GitCommitDate:   gitCommitDate,
 		GitTreeState:    gitTreeState,
 		BuildDate:       buildDate,
 		GoVersion:       runtime.Version(),
@@ -149,9 +194,11 @@ func Get() *Info {
 		Platform:        fmt.Sprintf("%s/%s", runtime.GOOS, runtime.GOARCH),
 		Linkmode:        linkmode,
 		BuildTags:       buildTags,
+		LDFlags:         ldFlags,
 		SeccompEnabled:  seccomp.IsEnabled(),
 		AppArmorEnabled: apparmor.IsEnabled(),
-	}
+		Dependencies:    dependencies,
+	}, nil
 }
 
 // String returns the string representation of the version info
@@ -173,7 +220,8 @@ func (i *Info) String() string {
 		case reflect.Slice:
 			// Only expecting []string here; ignore other slices.
 			if s, ok := value.Interface().([]string); ok {
-				valueString = strings.Join(s, ", ")
+				const sep = "\n  "
+				valueString = sep + strings.Join(s, sep)
 			}
 
 		case reflect.String:
