@@ -10,8 +10,8 @@ import (
 
 	"github.com/containers/buildah/util"
 	"github.com/containers/common/pkg/subscriptions"
-	"github.com/containers/podman/v3/pkg/rootless"
-	selinux "github.com/containers/podman/v3/pkg/selinux"
+	"github.com/containers/podman/v4/pkg/rootless"
+	selinux "github.com/containers/podman/v4/pkg/selinux"
 	cstorage "github.com/containers/storage"
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/containers/storage/pkg/mount"
@@ -28,7 +28,6 @@ import (
 	securejoin "github.com/cyphar/filepath-securejoin"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
@@ -48,21 +47,21 @@ func (s *Server) createContainerPlatform(ctx context.Context, container *oci.Con
 		rootPair := idMappings.RootPair()
 		for _, path := range []string{container.BundlePath(), container.MountPoint()} {
 			if err := makeAccessible(path, rootPair.UID, rootPair.GID, false); err != nil {
-				return errors.Wrapf(err, "cannot make %s accessible to %d:%d", path, rootPair.UID, rootPair.GID)
+				return fmt.Errorf("cannot make %s accessible to %d:%d: %w", path, rootPair.UID, rootPair.GID, err)
 			}
 		}
 		if err := makeMountsAccessible(rootPair.UID, rootPair.GID, container.Spec().Mounts); err != nil {
 			return err
 		}
 	}
-	return s.Runtime().CreateContainer(ctx, container, cgroupParent)
+	return s.Runtime().CreateContainer(ctx, container, cgroupParent, false)
 }
 
 // makeAccessible changes the path permission and each parent directory to have --x--x--x
 func makeAccessible(path string, uid, gid int, doChown bool) error {
 	if doChown {
 		if err := os.Chown(path, uid, gid); err != nil {
-			return errors.Wrapf(err, "cannot chown %s to %d:%d", path, uid, gid)
+			return fmt.Errorf("cannot chown %s to %d:%d: %w", path, uid, gid, err)
 		}
 	}
 	for ; path != "/"; path = filepath.Dir(path) {
@@ -116,7 +115,7 @@ func toContainer(id uint32, idMap []idtools.IDMap) uint32 {
 }
 
 // finalizeUserMapping changes the UID, GID and additional GIDs to reflect the new value in the user namespace.
-func (s *Server) finalizeUserMapping(specgen *generate.Generator, mappings *idtools.IDMappings) {
+func (s *Server) finalizeUserMapping(sb *sandbox.Sandbox, specgen *generate.Generator, mappings *idtools.IDMappings) {
 	if mappings == nil {
 		return
 	}
@@ -126,6 +125,10 @@ func (s *Server) finalizeUserMapping(specgen *generate.Generator, mappings *idto
 		return
 	}
 
+	if sb.Annotations()[crioann.UsernsModeAnnotation] == "" {
+		return
+	}
+
 	specgen.Config.Process.User.UID = toContainer(specgen.Config.Process.User.UID, mappings.UIDs())
 	gids := mappings.GIDs()
 	specgen.Config.Process.User.GID = toContainer(specgen.Config.Process.User.GID, gids)
@@ -327,10 +330,19 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 		return nil, err
 	}
 
+	annotationDevices, err := device.DevicesFromAnnotation(sb.Annotations()[crioann.DevicesAnnotation], s.config.AllowedDevices)
+	if err != nil {
+		return nil, err
+	}
+
+	if err := ctr.SpecAddDevices(configuredDevices, annotationDevices, privilegedWithoutHostDevices, s.config.DeviceOwnershipFromSecurityContext); err != nil {
+		return nil, err
+	}
+
 	s.resourceStore.SetStageForResource(ctr.Name(), "container storage start")
 	mountPoint, err := s.StorageRuntimeServer().StartContainer(containerID)
 	if err != nil {
-		return nil, fmt.Errorf("failed to mount container %s(%s): %v", containerName, containerID, err)
+		return nil, fmt.Errorf("failed to mount container %s(%s): %w", containerName, containerID, err)
 	}
 
 	defer func() {
@@ -344,16 +356,6 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 
 	s.resourceStore.SetStageForResource(ctr.Name(), "container spec configuration")
 
-	annotationDevices, err := device.DevicesFromAnnotation(sb.Annotations()[crioann.DevicesAnnotation], s.config.AllowedDevices)
-	if err != nil {
-		return nil, err
-	}
-
-	if err := ctr.SpecAddDevices(configuredDevices, annotationDevices, privilegedWithoutHostDevices, s.config.DeviceOwnershipFromSecurityContext); err != nil {
-		return nil, err
-	}
-	s.resourceStore.SetStageForResource(ctr.Name(), "container spec configuration")
-
 	labels := containerConfig.Labels
 
 	if err := validateLabels(labels); err != nil {
@@ -366,7 +368,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 			securityContext.ApparmorProfile,
 		)
 		if err != nil {
-			return nil, errors.Wrapf(err, "applying apparmor profile to container %s", containerID)
+			return nil, fmt.Errorf("applying apparmor profile to container %s: %w", containerID, err)
 		}
 
 		log.Debugf(ctx, "Applied AppArmor profile %s to container %s", profile, containerID)
@@ -411,7 +413,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 				specgen.SetLinuxResourcesMemoryLimit(memoryLimit)
 				if resources.MemorySwapLimitInBytes != 0 {
 					if resources.MemorySwapLimitInBytes > 0 && resources.MemorySwapLimitInBytes < resources.MemoryLimitInBytes {
-						return nil, errors.Errorf(
+						return nil, fmt.Errorf(
 							"container %s create failed because memory swap limit (%d) cannot be lower than memory limit (%d)",
 							ctr.ID(),
 							resources.MemorySwapLimitInBytes,
@@ -462,39 +464,13 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 		specgen.SetProcessNoNewPrivileges(securityContext.NoNewPrivs)
 
 		if !ctr.Privileged() {
-			for _, mp := range []string{
-				"/proc/acpi",
-				"/proc/kcore",
-				"/proc/keys",
-				"/proc/latency_stats",
-				"/proc/timer_list",
-				"/proc/timer_stats",
-				"/proc/sched_debug",
-				"/proc/scsi",
-				"/sys/firmware",
-				"/sys/dev",
-			} {
-				specgen.AddLinuxMaskedPaths(mp)
-			}
 			if securityContext.MaskedPaths != nil {
-				specgen.Config.Linux.MaskedPaths = nil
 				for _, path := range securityContext.MaskedPaths {
 					specgen.AddLinuxMaskedPaths(path)
 				}
 			}
 
-			for _, rp := range []string{
-				"/proc/asound",
-				"/proc/bus",
-				"/proc/fs",
-				"/proc/irq",
-				"/proc/sys",
-				"/proc/sysrq-trigger",
-			} {
-				specgen.AddLinuxReadonlyPaths(rp)
-			}
 			if securityContext.ReadonlyPaths != nil {
-				specgen.Config.Linux.ReadonlyPaths = nil
 				for _, path := range securityContext.ReadonlyPaths {
 					specgen.AddLinuxReadonlyPaths(path)
 				}
@@ -647,7 +623,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 			securityContext.Seccomp,
 			containerConfig.Linux.SecurityContext.SeccompProfilePath,
 		); err != nil {
-			return nil, errors.Wrap(err, "setup seccomp")
+			return nil, fmt.Errorf("setup seccomp: %w", err)
 		}
 	}
 
@@ -789,7 +765,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 	ociContainer.SetIDMappings(containerIDMappings)
 	var rootPair idtools.IDPair
 	if containerIDMappings != nil {
-		s.finalizeUserMapping(specgen, containerIDMappings)
+		s.finalizeUserMapping(sb, specgen, containerIDMappings)
 
 		for _, uidmap := range containerIDMappings.UIDs() {
 			specgen.AddLinuxUIDMapping(uint32(uidmap.HostID), uint32(uidmap.ContainerID), uint32(uidmap.Size))
@@ -806,7 +782,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 		}
 		for _, path := range pathsToChown {
 			if err := makeAccessible(path, rootPair.UID, rootPair.GID, true); err != nil {
-				return nil, errors.Wrapf(err, "cannot chown %s to %d:%d", path, rootPair.UID, rootPair.GID)
+				return nil, fmt.Errorf("cannot chown %s to %d:%d: %w", path, rootPair.UID, rootPair.GID, err)
 			}
 		}
 	} else if err := specgen.RemoveLinuxNamespace(string(rspec.UserNamespace)); err != nil {
@@ -820,7 +796,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 	// compatible with Docker
 	mtab := filepath.Join(mountPoint, "/etc/mtab")
 	if err := idtools.MkdirAllAs(filepath.Dir(mtab), 0o755, rootPair.UID, rootPair.GID); err != nil {
-		return nil, errors.Wrap(err, "error creating mtab directory")
+		return nil, fmt.Errorf("error creating mtab directory: %w", err)
 	}
 	if err := os.Symlink("/proc/mounts", mtab); err != nil && !os.IsExist(err) {
 		return nil, err
@@ -948,17 +924,26 @@ func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel,
 			src = resolvedSrc
 		} else {
 			if !os.IsNotExist(err) {
-				return nil, nil, fmt.Errorf("failed to resolve symlink %q: %v", src, err)
+				return nil, nil, fmt.Errorf("failed to resolve symlink %q: %w", src, err)
 			}
 			for _, toReject := range absentMountSourcesToReject {
 				if filepath.Clean(src) == toReject {
 					// special-case /etc/hostname, as we don't want it to be created as a directory
 					// This can cause issues with node reboot.
-					return nil, nil, errors.Errorf("Cannot mount %s: path does not exist and will cause issues as a directory", toReject)
+					return nil, nil, fmt.Errorf("cannot mount %s: path does not exist and will cause issues as a directory", toReject)
 				}
 			}
-			if err = os.MkdirAll(src, 0o755); err != nil {
-				return nil, nil, fmt.Errorf("failed to mkdir %s: %s", src, err)
+			if !ctr.Restore() {
+				// Although this would also be really helpful for restoring containers
+				// it is problematic as during restore external bind mounts need to be
+				// a file if the destination is a file. Unfortunately it is not easy
+				// to tell if the destination is a file or a directory. Especially if
+				// the destination is a nested bind mount. For now we will just not
+				// create the missing bind mount source for restore and return an
+				// error to the user.
+				if err = os.MkdirAll(src, 0o755); err != nil {
+					return nil, nil, fmt.Errorf("failed to mkdir %s: %s", src, err)
+				}
 			}
 		}
 
@@ -1004,8 +989,6 @@ func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel,
 			} else if err := securityLabel(src, mountLabel, false, maybeRelabel); err != nil {
 				return nil, nil, err
 			}
-		} else {
-			log.Debugf(ctx, "Skipping relabel for %s because kubelet did not request it", src)
 		}
 
 		volumes = append(volumes, oci.ContainerVolume{
