@@ -4,10 +4,10 @@ import (
 	"context"
 
 	"github.com/cri-o/cri-o/internal/oci"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
-	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // The actual test suite
@@ -29,25 +29,23 @@ var _ = t.Describe("ContainerStop", func() {
 			})
 
 			// When
-			response, err := sut.StopContainer(context.Background(),
-				&pb.StopContainerRequest{
+			err := sut.StopContainer(context.Background(),
+				&types.StopContainerRequest{
 					ContainerId: testContainer.ID(),
 				})
 
 			// Then
 			Expect(err).To(BeNil())
-			Expect(response).NotTo(BeNil())
 		})
 
-		It("should afil with invalid container id", func() {
+		It("should fail with invalid container id", func() {
 			// Given
 			// When
-			response, err := sut.StopContainer(context.Background(),
-				&pb.StopContainerRequest{ContainerId: "id"})
+			err := sut.StopContainer(context.Background(),
+				&types.StopContainerRequest{ContainerId: "id"})
 
 			// Then
 			Expect(err).NotTo(BeNil())
-			Expect(response).To(BeNil())
 		})
 	})
 })
