@@ -6,8 +6,11 @@ import (
 	pb "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
-func (c *service) StartContainer(
+func (s *service) StartContainer(
 	ctx context.Context, req *pb.StartContainerRequest,
 ) (resp *pb.StartContainerResponse, retErr error) {
-	return nil, nil
+	if err := s.server.StartContainer(ctx, req); err != nil {
+		return nil, err
+	}
+	return &pb.StartContainerResponse{}, nil
 }
