@@ -2,19 +2,16 @@ package v1alpha2
 
 import (
 	"context"
+	"unsafe"
 
-	"github.com/cri-o/cri-o/server/cri/types"
+	v1 "k8s.io/cri-api/pkg/apis/runtime/v1"
 	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
 )
 
 func (s *service) StopContainer(
 	ctx context.Context, req *pb.StopContainerRequest,
 ) (*pb.StopContainerResponse, error) {
-	r := &types.StopContainerRequest{
-		ContainerID: req.ContainerId,
-		Timeout:     req.Timeout,
-	}
-	if err := s.server.StopContainer(ctx, r); err != nil {
+	if err := s.server.StopContainer(ctx, (*v1.StopContainerRequest)(unsafe.Pointer(req))); err != nil {
 		return nil, err
 	}
 	return &pb.StopContainerResponse{}, nil
